@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #define BORENDERMAIN_H
 
 #include "bodebugdcopiface.h"
-#include "bosonglwidget.h"
+#include "bosonufoglwidget.h"
 
 #include <qptrlist.h>
 #include <qptrdict.h>
@@ -44,13 +44,20 @@ class BoCamera;
 class BoCameraWidget;
 class BoMatrix;
 class BoQuaternion;
-class BoVector3;
 class BoFrame;
 class BoLight;
 class BoLightCameraWidget;
+class BoLightCameraWidget1;
 class BoMaterialWidget;
 class KCmdLineArgs;
 class QCheckBox;
+class BoFontInfo;
+class BoUfoManager;
+class BoUfoLabel;
+class BoUfoAction;
+class BosonViewData;
+template<class T> class BoVector3;
+typedef BoVector3<float> BoVector3Float;
 
 class KMyFloatNumInput : public KDoubleNumInput
 {
@@ -91,17 +98,19 @@ private slots:
 /**
  * @author Andreas Beckermann <b_mann@gmx.de>
  **/
-class ModelPreview : public BosonGLWidget
+class ModelPreview : public BosonUfoGLWidget
 {
 	Q_OBJECT
 public:
-	ModelPreview(QWidget*);
+	ModelPreview(const QPtrList<SpeciesTheme>&, QWidget*);
 	~ModelPreview();
 
 	virtual void initializeGL();
 	virtual void paintGL();
 	virtual void resizeGL(int, int);
 
+	void setFont(const BoFontInfo& font);
+
 	void load(SpeciesTheme* s, const UnitProperties* prop);
 	void loadObjectModel(SpeciesTheme* s, const QString& file);
 	void resetModel();
@@ -110,6 +119,13 @@ public:
 	BoLight* light() const { return mLight; }
 	BosonModel* model() const { return mModel; }
 
+	void changeUnit(const QString& speciesIdentifier, const QString& unit);
+	void changeUnit(const QString& speciesIdentifier, unsigned long int unitType);
+	void changeObject(const QString& speciesIdentifier, const QString& file);
+
+	void changeUnit(SpeciesTheme* s, const UnitProperties* prop);
+	void changeObject(SpeciesTheme* s, const QString& file);
+
 signals:
 	void signalFovYChanged(float);
 
@@ -122,13 +138,32 @@ signals:
 	void signalCameraChanged();
 
 	void signalFrameChanged(int);
+	void signalFrameChanged(float);
 	void signalLODChanged(int);
+	void signalLODChanged(float);
 
 	void signalMaxFramesChanged(int);
+	void signalMaxFramesChanged(float);
 	void signalMaxLODChanged(int);
+	void signalMaxLODChanged(float);
 
 	void signalMeshSelected(int);
 
+
+	void signalChangeBackgroundColor();
+	void signalShowLightWidget();
+	void signalShowMaterialsWidget();
+	void signalShowChangeFont();
+	void signalDebugModels();
+	void signalDebugMemory();
+	void signalDebugSpecies();
+	void signalShowGLStates();
+	void signalReloadModelTextures();
+	void signalReloadMeshRenderer();
+
+	void signalUnitChanged(SpeciesTheme*, int);
+	void signalObjectChanged(SpeciesTheme*, int);
+
 public slots:
 	void slotResetView();
 	void slotFovYChanged(float f)
@@ -138,7 +173,15 @@ public slots:
 			resizeGL(width(), height());
 		}
 	}
+	void slotFrameChanged(float f)
+	{
+		slotFrameChanged((int)f);
+	}
 	void slotFrameChanged(int f);
+	void slotLODChanged(float l)
+	{
+		slotLODChanged((int)l);
+	}
 	void slotLODChanged(int l);
 	void slotPlacementPreviewChanged(bool on)
 	{
@@ -152,7 +195,6 @@ public slots:
 	{
 		mWireFrame = on;
 	}
-	void slotConstructionChanged(bool on);
 	void slotRenderAxisChanged(bool on)
 	{
 		mRenderAxis = on;
@@ -165,6 +207,28 @@ public slots:
 	void slotHideSelectedMesh();
 	void slotHideUnSelectedMeshes();
 	void slotUnHideAllMeshes();
+	void slotEnableLight(bool);
+	void slotEnableMaterials(bool);
+	void slotShowVertexPoints(bool);
+	void slotChangeVertexPointSize();
+	void slotChangeGridUnitSize();
+
+	void slotUnitChanged(int);
+	void slotObjectChanged(int);
+
+	void slotSetVertexPointSize(int);
+	void slotSetGridUnitSize(float);
+
+	void slotChangeBackgroundColor();
+	void slotShowLightWidget();
+	void slotDebugModels();
+	void slotDebugMemory();
+	void slotDebugSpecies();
+	void slotShowMaterialsWidget();
+	void slotShowGLStates();
+	void slotReloadModelTextures();
+	void slotReloadMeshRenderer();
+	void slotShowChangeFont();
 
 protected:
 	virtual bool eventFilter(QObject* o, QEvent* e);
@@ -188,11 +252,12 @@ protected:
 	}
 
 	void setModel(BosonModel*);
-	BoFrame* frame(unsigned int f) const;
 
-	void updateCamera(const BoVector3& cameraPos, const BoQuaternion& q);
-	void updateCamera(const BoVector3& cameraPos, const BoMatrix& rotationMatrix);
-	void updateCamera(const BoVector3& cameraPos, const BoVector3& lookAt, const BoVector3& up);
+	void updateCamera(const BoVector3Float& cameraPos, const BoQuaternion& q);
+	void updateCamera(const BoVector3Float& cameraPos, const BoMatrix& rotationMatrix);
+	void updateCamera(const BoVector3Float& cameraPos, const BoVector3Float& lookAt, const BoVector3Float& up);
+
+	void updateMeshUnderMouseLabel();
 
 	/**
 	 * Use -1 to select nothing.
@@ -212,6 +277,14 @@ protected:
 	void updateCursorDisplay(const QPoint& pos);
 	int pickObject(const QPoint& pos);
 
+	void uncheckAllBut(BoUfoAction*); // BAH!
+
+	SpeciesTheme* findTheme(const QString& theme) const;
+
+private:
+	void initUfoGUI();
+	void initUfoAction();
+
 private:
 	friend class RenderMain; // we need to emit signals from outside, in order to save lots of forwarding code
 	QTimer* mUpdateTimer;
@@ -220,6 +293,9 @@ private:
 	int mCurrentLOD;
 	int mMeshUnderMouse;
 	int mSelectedMesh;
+	BoUfoLabel* mMeshUnderMouseLabel;
+	BoUfoLabel* mSelectedMeshLabel;
+	BosonViewData* mViewData;
 
 	BosonGLFont* mDefaultFont;
 
@@ -228,7 +304,6 @@ private:
 	bool mPlacementPreview;
 	bool mDisallowPlacement;
 	bool mWireFrame;
-	bool mConstruction;
 	bool mRenderAxis;
 	bool mRenderGrid;
 
@@ -236,65 +311,18 @@ private:
 
 	BoCamera* mCamera;
 	BoLight* mLight;
-};
 
-class PreviewConfig : public QWidget
-{
-	Q_OBJECT
-public:
-	PreviewConfig(QWidget* parent);
-	~PreviewConfig();
+	QPtrList<SpeciesTheme> mSpecies;
+	QPtrDict<SpeciesTheme> mAction2Species;
 
-	void setCamera(BoCamera* camera);
-
-signals:
-	void signalFovYChanged(float);
-	void signalFrameChanged(int);
-	void signalLODChanged(int);
-	void signalResetDefaults();
-	void signalPlacementPreviewChanged(bool); // display preview placement - if false display normal model
-	void signalDisallowPlacementChanged(bool); // only valid of placementpreview is also on. if true display the model that is shown when the unit can't be placed - otherwise the model that is shown if it can be placed.
-	void signalWireFrameChanged(bool);
-	void signalConstructionChanged(bool);
-	void signalRenderAxisChanged(bool);
-	void signalRenderGridChanged(bool);
-	void signalHideMesh();
-	void signalHideOthers();
-	void signalUnHideAll();
-
-public slots:
-	void slotCameraChanged();
-	void slotMeshSelected(int);
-
-protected slots:
-	void slotFovYChanged(float f) { mFovY->setValue(f); }
-	void slotFrameChanged(int f) { mFrame->setValue(f); }
-	void slotLODChanged(int l) { mFrame->setValue(l); }
-
-	void slotMaxFramesChanged(int max) { mFrame->setRange(0, max); }
-	void slotMaxLODChanged(int max) { mLOD->setRange(0, max); }
-	void slotEnableLightChanged(bool);
-	void slotEnableMaterialsChanged(bool);
-
-private:
-	KMyFloatNumInput* mFovY;
-	KIntNumInput* mFrame;
-	KIntNumInput* mLOD;
-	QPushButton* mHideMesh;
-	QPushButton* mHideOthers;
-	QPushButton* mUnHideAll;
-	QCheckBox* mPlacementPreview;
-	QCheckBox* mDisallowPlacement;
-	QCheckBox* mWireFrame;
-	QCheckBox* mConstruction;
-	BoCameraWidget* mCameraWidget;
-	QCheckBox* mRenderAxis;
-	QCheckBox* mRenderGrid;
-	QCheckBox* mEnableLight;
-	QCheckBox* mEnableMaterials;
+	// TODO: use a BoUfo widget. don't use a separate window.
+	BoLightCameraWidget1* mLightWidget;
+	BoMaterialWidget* mMaterialWidget;
 };
 
 /**
+ * This widget is here mainly for historic reasons. It just contains a @ref
+ * Modelpreview widget. Most interesting things are done there.
  * @author Andreas Beckermann <b_mann@gmx.de>
  **/
 class RenderMain : public KMainWindow
@@ -314,52 +342,13 @@ public:
 	void emitSignalFrame(int f) { emit mPreview->signalFrameChanged(f); }
 	void emitSignalLOD(int l) { emit mPreview->signalLODChanged(l); }
 
-protected:
-	void initKAction();
-
-protected:
-	/**
-	 * Connect both objects to the signal. Both objects need to provide the
-	 * signal and the slot (i.e. it must have the same name/params)
-	 **/
-	void connectBoth(QObject* o1, QObject* o2, const char* signal, const char* slot);
-
-	/**
-	 * Display another unit. This will load the unit from the @ref
-	 * ModelPreview. See @ref ModelPreview::load
-	 **/
-	void changeUnit(SpeciesTheme* s, const UnitProperties* prop);
-
-	void changeObject(SpeciesTheme* s, const QString& file);
-
-	SpeciesTheme* findTheme(const QString& theme) const;
-	void uncheckAllBut(KAction*); // BAH!
-
-protected slots:
-	void slotUnitChanged(int);
-	void slotObjectChanged(int);
-	void slotDebugModels();
-	void slotDebugSpecies();
-	void slotVertexPointSize();
-	void slotGridUnitSize();
-	void slotShowVertexPoints(bool);
-	void slotBackgroundColor();
-	void slotShowLightWidget();
-	void slotShowMaterialsWidget();
-	void slotShowGLStates();
-	void slotReloadModelTextures();
-	void slotReloadMeshRenderer();
-
 private:
-	PreviewConfig* mConfig;
 	ModelPreview* mPreview;
 	QPtrList<SpeciesTheme> mSpecies;
-	QPtrDict<SpeciesTheme> mAction2Species;
 	BoDebugDCOPIface* mIface;
-
-	BoLightCameraWidget* mLightWidget;
-	BoMaterialWidget* mMaterialWidget;
 };
 
+
+
 #endif
 
