@@ -1,6 +1,7 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +22,7 @@
 
 #include "bosoncollisions.h"
 #include "defines.h"
+#include "bomath.h"
 
 #include <qobject.h>
 
@@ -32,11 +34,17 @@ class UnitProperties;
 class BoItemList;
 class BosonItem;
 class ProductionPlugin;
-class BosonEffect;
 class BosonShot;
+class BosonWeapon;
 class BosonCanvasStatistics;
-class BoVector3;
-class BosonPath2;
+class BosonPath;
+class BosonMoveData;
+class BosonShotFragment;
+class BoEventListener;
+template<class T> class BoVector2;
+template<class T> class BoVector3;
+typedef BoVector2<bofixed> BoVector2Fixed;
+typedef BoVector3<bofixed> BoVector3Fixed;
 
 class KPlayer;
 class QDataStream;
@@ -45,6 +53,7 @@ template<class T> class QPtrList;
 template<class T> class QValueList;
 template<class T> class QPtrVector;
 template<class T1, class T2> class QMap;
+template<class T1, class T2> class QPair;
 
 /**
  * Helper class for @ref BosonCanvas::createItem.
@@ -144,20 +153,8 @@ public:
 
 	BosonCanvasStatistics* canvasStatistics() const;
 
-	/**
-	 * Create the @ref Cell array
-	 **/
-	void createCells(int w, int h);
-
-	/**
-	 * Initialize this @ref Cell.
-	 *
-	 * All this currently does is to set the tile. See @ref QCanvas::setTile
-	 **/
-	void initCell(int x, int y);
-
 	void initPathfinder();
-	BosonPath2* pathfinder();
+	BosonPath* pathfinder() const;
 
 	inline BosonCollisions* collisions() const { return mCollisions; }
 
@@ -167,14 +164,15 @@ public:
 	 * This will also do the necessary steps for newly added items, such as
 	 * loading unit defaults and adding the unit to the @p owner.
 	 **/
-	BosonItem* createNewItem(int rtti, Player* owner, const ItemType& type, const BoVector3& pos);
+	BosonItem* createNewItem(int rtti, Player* owner, const ItemType& type, const BoVector3Fixed& pos);
 
 	/**
 	 * Test whether the unit can go over rect. This method only tests for
-	 * the ground (see @ref Cell) <em>not</em> for collisions with other
-	 * units. See @ref Unit for this.
+	 * the ground <em>not</em> for collisions with other units. See @ref Unit for
+	 * this.
 	 **/
-	bool canGo(const UnitProperties* prop, const QRect& rect) const;
+	bool canGo(const UnitProperties* prop, const BoRectFixed& rect, bool _default = false) const;
+	bool canGo(const UnitProperties* prop, int x, int y, bool _default = false) const;
 
 	void setMap(BosonMap* map);
 	BosonMap* map() const;
@@ -199,21 +197,34 @@ public:
 	 **/
 	void setHeightAtCorner(int x, int y, float height);
 
+	/**
+	 * See @ref BosonMap::setHeightsAtCorners
+	 **/
+	void setHeightsAtCorners(const QValueList< QPair<QPoint, float> >& heights);
+
 	/**
 	 * @return BosonMap::heightAtCorner
 	 **/
 	float heightAtCorner(int x, int y) const;
 
+	/**
+	 * @return BosonMap::waterDepthAtCorner
+	 **/
+	float waterDepthAtCorner(int x, int y) const;
+
 	/**
 	 * @return Height at point x,y (in canvas coordinates)
 	 * This is calculated from heights at the corners of the cell that the point
-	 * is on
+	 * is on. Water surface level is also taken insto account
 	 **/
-	float heightAtPoint(float x, float y) const;
+	float heightAtPoint(bofixed x, bofixed y) const;
 
-	void addAnimation(BosonItem* item);
-	void removeAnimation(BosonItem* item);
-	unsigned int animationsCount() const;
+	/**
+	 * @return Height at point x,y (in canvas coordinates)
+	 * This is calculated from heights at the corners of the cell that the point
+	 *  is on. Note that this method _does not_ take water level into account.
+	 **/
+	float terrainHeightAtPoint(bofixed x, bofixed y) const;
 
 	/**
 	 * Request to change the advance list in the next advance phase.
@@ -245,7 +256,7 @@ public:
 	 *
 	 * Also adjust the mini map - see @ref signalUnitMoved
 	 **/
-	void unitMoved(Unit* unit, float oldX, float oldY);
+	void unitMoved(Unit* unit, bofixed oldX, bofixed oldY);
 
 	void unitMovingStatusChanges(Unit* u, int oldstatus, int newstatus);
 
@@ -268,7 +279,7 @@ public:
 	 * @param range Radius of explosion. All units range or less cells away will be damaged
 	 * @param owner Player who caused the explosion. Used for statistics. May be null
 	 **/
-	void explosion(const BoVector3& pos, long int damage, float range, float fullrange, Player* owner);
+	void explosion(const BoVector3Fixed& pos, long int damage, bofixed range, bofixed fullrange, Player* owner);
 
 	/**
 	 * Called when unit is damaged (usually by missile).
@@ -283,6 +294,17 @@ public:
 	 **/
 	void destroyUnit(Unit* unit);
 
+	/**
+	 * @internal
+	 * Used by @ref BosonWeapon::shoot
+	 *
+	 * This causes the correct effects (visual as well as sound effects) to
+	 * be emitted
+	 *
+	 * See also @ref signalShotFired.
+	 **/
+	void shotFired(BosonShot* shot, BosonWeapon* weapon);
+
 	/**
 	 * Prepare the unit to be deleted. Remove the unit from the player and
 	 * so on. This doesn't play any sound or so, so it can be used in
@@ -296,14 +318,14 @@ public:
 	 **/
 	void removeUnit(Unit* unit);
 
-	void updateSight(Unit*, float oldX, float oldY);
+	void updateSight(Unit*, bofixed oldX, bofixed oldY);
 
 	Cell* cellAt(Unit* unit) const;
 
 	/**
 	 * @return The cell at @p x, @p y in <em>canvas</em>-coordinates
 	 **/
-	Cell* cellAt(float x, float y) const;
+	Cell* cellAt(bofixed x, bofixed y) const;
 
 	/**
 	 * @return The cell at @p x, @p y (in cell coordinates - see @ref cellAt
@@ -319,25 +341,27 @@ public:
 	void deleteDestroyed();
 	void deleteUnusedShots();
 
+	BosonItem* findItem(unsigned long int id) const;
+
 	/**
 	 * Convenience method. See @ref BosonCollisions::findUnitAt
 	 **/
-	Unit* findUnitAt(const BoVector3& pos) const { return collisions()->findUnitAt(pos); }
+	Unit* findUnitAt(const BoVector3Fixed& pos) const { return collisions()->findUnitAt(pos); }
 
 	/**
 	 * Convenience method. See @ref BosonCollisions::findItemAt
 	 **/
-	BosonItem* findItemAt(const BoVector3& pos) const { return collisions()->findItemAt(pos); }
+	BosonItem* findItemAt(const BoVector3Fixed& pos) const { return collisions()->findItemAt(pos); }
 
 	/**
 	 * Convenience method. See @ref BosonCollisions::findUnitAtCell
 	 **/
-	Unit* findUnitAtCell(int x, int y, float z) const { return collisions()->findUnitAtCell(x, y, z); }
+	Unit* findUnitAtCell(int x, int y, bofixed z) const { return collisions()->findUnitAtCell(x, y, z); }
 
 	/**
 	 * Convenience method. See @ref BosonCollisions::findItemAtCell
 	 **/
-	BosonItem* findItemAtCell(int x, int y, float z, bool unitOnly) const { return collisions()->findItemAtCell(x, y, z, unitOnly); }
+	BosonItem* findItemAtCell(int x, int y, bofixed z, bool unitOnly) const { return collisions()->findItemAtCell(x, y, z, unitOnly); }
 
 	/**
 	 * Convenience method. See @ref BosonCollisions::cellOccupied
@@ -352,78 +376,105 @@ public:
 	/**
 	 * Checks if any items, except exclude, collide with given box
 	 **/
-	QValueList<Unit*> collisionsInBox(const BoVector3& v1, const BoVector3& v2, BosonItem* exclude) const { return collisions()->collisionsInBox(v1, v2, exclude); }
+	QValueList<Unit*> collisionsInBox(const BoVector3Fixed& v1, const BoVector3Fixed& v2, BosonItem* exclude) const { return collisions()->collisionsInBox(v1, v2, exclude); }
 
 	/**
 	 * @param pos The location where the unit should get placed.
-	 * This is in <em>cell</em>-coordinates. This point specifies the
-	 * <em>upper-left</em> corner of the unit.
+	 * This point specifies the <em>upper-left</em> corner of the unit.
 	 * @param factory If NULL then BUILD_RANGE is ignored. Otherwise
 	 * facilities must be in range of BUILD_RANGE of any player unit and
 	 * mobile units in BUILD_RANGE of the facility.
 	 * @return TRUE if the unit can be placed at pos, otherwise FALSE
 	 **/
-	bool canPlaceUnitAtCell(const UnitProperties* unit, const QPoint& pos, ProductionPlugin* factory) const;
+	bool canPlaceUnitAt(const UnitProperties* unit, const BoVector2Fixed& pos, ProductionPlugin* factory) const;
 
 	/**
-	 * Same as above, but this takes canvas coordinates in @p pos.
+	 * Clear the canvas, preparing it for destruction.
 	 *
-	 * Note that this is <em>not</em> more accurate than the above one. It
-	 * does not check for half-cells or so, i.e. does not check for pixel
-	 * positions. It simply checks for a valid canvas position as well
-	 * (this is usefule, since (-10,-10) on canvas maps to (0,0), but it
-	 * should be invalid!)
+	 * This method is meant to eliminate all pointers and all allocated
+	 * memory from the canvas. For notifying the game that a player has won,
+	 * see @ref signalGameOver.
 	 **/
-	bool canPlaceUnitAt(const UnitProperties* unit, const QPoint& pos, ProductionPlugin* factory) const
-	{
-		if (!onCanvas(pos)) {
-			return false;
-		}
-		return canPlaceUnitAtCell(unit, pos / BO_TILE_SIZE, factory);
-	}
-
 	void quitGame();
 
 	void addToCells(BosonItem* u);
 	void removeFromCells(BosonItem* u);
 
-	bool onCanvas(const QPoint& pos) const
+	bool onCanvas(const BoVector2Fixed& pos) const;
+	bool onCanvas(const BoVector3Fixed& pos) const;
+	bool onCanvas(bofixed x, bofixed y) const
 	{
-		return onCanvas(pos.x(), pos.y());
-	}
-	bool onCanvas(const BoVector3& canvasPos) const;
-	bool onCanvas(int x, int y) const
-	{
-		return x >= 0 && y >= 0 && (unsigned int)x < mapWidth() * BO_TILE_SIZE && (unsigned int)y < mapHeight() * BO_TILE_SIZE;
+		return x >= 0.0f && y >= 0.0f && x <= mapWidth() && y <= mapHeight();
 	}
 
 	bool advanceFunctionLocked() const { return mAdvanceFunctionLocked; }
 
-	unsigned int effectsCount() const;
-	void updateEffects(float elapsed);
-	QPtrList<BosonEffect>* effects() const;
-	void addEffect(BosonEffect* e);
-	void addEffects(const QPtrList<BosonEffect> effects);
-
 	bool loadFromXML(const QDomElement& root);
-	bool saveAsXML(QDomElement& root);
+	bool saveAsXML(QDomElement& root) const;
+
+	/**
+	 * Load the conditions for the @ref BoCanvasEventListener (i.e. the
+	 * global @ref BoEventListener) from @p root.
+	 *
+	 * Note that the conditions are loaded <em>only</em>, not the whole
+	 * event listener.
+	 *
+	 * See also @ref BoEventListener::loadConditions
+	 **/
+	bool loadConditions(const QDomElement& root);
+
+	/**
+	 * Save the conditions from the @ref BoCanvasEventListener to @p root.
+	 * This @ref QDomElement can be used for loading the conditions using
+	 * @ref loadConditions again.
+	 *
+	 * See also @ref BoEventListener::saveConditions
+	 **/
+	bool saveConditions(QDomElement& root) const;
 
 	/**
 	 * Valid in editor mode only. Delete the specified items.
 	 **/
 	void deleteItems(const QValueList<unsigned long int>& items);
 
+	BoEventListener* eventListener() const;
+
+	/**
+	 * @internal
+	 * Used by @ref BosonPath
+	 **/
+	void clearMoveDatas();
+
+	void insertMoveData(const UnitProperties* prop, BosonMoveData* data);
+	BosonMoveData* moveData(const UnitProperties* prop) const;
+
 public slots:
 	/**
 	 * @param See @ref Boson::signalAdvance
 	 **/
-	void slotAdvance(unsigned int advanceCount, bool advanceFlag);
+	void slotAdvance(unsigned int advanceCallsCount, bool advanceFlag);
 
 signals:
 	void signalItemAdded(BosonItem* item);
-	void signalUnitMoved(Unit* unit, float oldX, float oldY);
+	void signalUnitMoved(Unit* unit, bofixed oldX, bofixed oldY);
 	void signalUnitRemoved(Unit* unit);
 
+	/**
+	 * Like @ref signalUnitRemoved, but the unit actually got destroyed
+	 * (e.g. this is not emitted in editor mode)
+	 **/
+	void signalUnitDestroyed(Unit* unit);
+
+	/**
+	 * Signal to notify the GUI that a fragment was created. A corresponding
+	 * effect can be created in a GUI class now.
+	 *
+	 * Note that this signal is ugly. However fragments need additional
+	 * initialization after their creation, therefore @ref signalItemAdded
+	 * can not be used directly.
+	 **/
+	void signalFragmentCreated(BosonShotFragment* fragment);
+
 	/**
 	 * Emitted by @ref removeItem just after the item is removed from the
 	 * canvas. Note that this usually happens from the destructor of an
@@ -440,6 +491,19 @@ signals:
 	 **/
 	void signalRemovedItem(BosonItem* item);
 
+	void signalShotFired(BosonShot* shot, BosonWeapon* weapon);
+	void signalShotHit(BosonShot* shot);
+
+	/**
+	 * Emitted when the winning conditions are fullfilled. See @ref
+	 * BoCanvasEventListener.
+	 *
+	 * @param fullfilledWinningConditions The list of players that have
+	 * completed their goals of this scenario, i.e. the players that have
+	 * won. If this list is empty, no player has won.
+	 **/
+	void signalGameOver();
+
 protected:
 	/**
 	 * This is a minimalistic method for creating @ref BosonItem objects.
@@ -458,7 +522,7 @@ protected:
 	 *
 	 * @param id A unique ID for the item. See @ref BosonItem::setId
 	 **/
-	BosonItem* createItem(int rtti, Player* owner, const ItemType& type, const BoVector3& pos, unsigned long int id);
+	BosonItem* createItem(int rtti, Player* owner, const ItemType& type, const BoVector3Fixed& pos, unsigned long int id);
 
 	Unit* createUnit(Player* owner, unsigned long int unitType);
 	BosonShot* createShot(Player* owner, unsigned long int shotType, unsigned long int unitType, unsigned long int weaponPropertyId);
@@ -469,9 +533,10 @@ protected:
 	 * Used by @ref loadFromXML.
 	 **/
 	bool loadItemsFromXML(const QDomElement& root);
-#if 0
-	bool loadEffectsFromXML(const QDomElement& root);
-#endif
+	bool saveItemsAsXML(QDomElement& root) const;
+
+	bool loadEventListenerFromXML(const QDomElement& root);
+	bool saveEventListenerAsXML(QDomElement& root) const;
 
 	/**
 	 * Parses @p item and will create a @ref BosonItem correspoding to the
@@ -512,6 +577,7 @@ private:
 	void init();
 
 private:
+	friend class BoCanvasAdvance;
 	class BosonCanvasPrivate;
 	BosonCanvasPrivate* d;
 	BosonCollisions* mCollisions;
