@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,9 +19,16 @@
 
 #include "bosonfileconverter.h"
 
+#include "../bomemory/bodummymemory.h"
 #include "bodebug.h"
 #include "defines.h"
 #include "boversion.h"
+#include "bomath.h"
+
+// do NOT include unit.h, rtti.h, unitbase.h, player.h, ... here!
+// -> you should hardcode everything. all values and methods from these files
+//    can change in different versions, so making a file converter depend on
+//    them is completely nonsense!
 
 #include <qdatastream.h>
 #include <qcolor.h>
@@ -29,6 +36,10 @@
 #include <qdom.h>
 #include <qimage.h>
 #include <qstringlist.h>
+#include <qmap.h>
+
+#include <math.h>
+
 
 // version number as used by boson 0.8.128 (aka 0x00,0x08,0x80 - development
 // version. got never released)
@@ -641,6 +652,697 @@ bool BosonFileConverter::convertPlayField_From_0_9_1_To_0_10(QMap<QString, QByte
  return true;
 }
 
+bool BosonFileConverter::convertPlayField_From_0_10_To_0_10_80(QMap<QString, QByteArray>& files)
+{
+ QDomDocument kgameDoc(QString::fromLatin1("Boson"));
+ QDomDocument canvasDoc(QString::fromLatin1("Canvas"));
+ QDomDocument playersDoc(QString::fromLatin1("Players"));
+ if (!loadXMLDoc(&kgameDoc, files["kgame.xml"])) {
+	boError() << k_funcinfo << "could not load kgame.xml" << endl;
+	return false;
+ }
+ if (!loadXMLDoc(&canvasDoc, files["canvas.xml"])) {
+	boError() << k_funcinfo << "could not load canvas.xml" << endl;
+	return false;
+ }
+ if (!loadXMLDoc(&playersDoc, files["players.xml"])) {
+	boError() << k_funcinfo << "could not load players.xml" << endl;
+	return false;
+ }
+ QDomElement kgameRoot = kgameDoc.documentElement();
+ QDomElement canvasRoot = canvasDoc.documentElement();
+ QDomElement playersRoot = playersDoc.documentElement();
+
+ // 0.10.80 is a development version that was never released.
+ // this just exist because we have some maps in cvs with this format version
+ // and need to convert them, too.
+ kgameRoot.setAttribute("Version", BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x04));
+
+ // all occurances of the player ID/index are now stored in an attribute named
+ // "PlayerId"
+ QDomNodeList playersList = playersRoot.elementsByTagName("Player");
+ if (playersList.count() < 2) {
+	boError() << k_funcinfo << "less than 2 Player tags found in file. This is an invalid file." << endl;
+	return false;
+ }
+ for (unsigned int i = 0; i < playersList.count(); i++) {
+	QDomElement e = playersList.item(i).toElement();
+	if (e.isNull()) {
+		boError() << k_funcinfo << "invalid Player tag" << endl;
+		return false;
+	}
+	QString id = e.attribute("Id");
+	e.removeAttribute("Id");
+	e.setAttribute("PlayerId", id);
+	if (i == playersList.count() - 1) {
+		if (!e.hasAttribute("IsNeutral")) {
+			boError() << k_funcinfo << "file format error: last player must be neutral player" << endl;
+			return false;
+		}
+		bool ok = false;
+		if (e.attribute("IsNeutral").toUInt(&ok) != 1 || !ok) {
+			boError() << k_funcinfo << "IsNeutral attribute must be 1 if present" << endl;
+			return false;
+		}
+
+	}
+ }
+ QDomNodeList itemsList = canvasRoot.elementsByTagName("Items");
+ for (unsigned int i = 0; i < itemsList.count(); i++) {
+	QDomElement e = itemsList.item(i).toElement();
+	if (e.isNull()) {
+		boError() << k_funcinfo << "invalid Items tag" << endl;
+		return false;
+	}
+	QString id = e.attribute("Id");
+	e.removeAttribute("Id");
+	e.setAttribute("PlayerId", id);
+ }
+
+
+ // events/conditions:
+ QDomElement eventManager = kgameDoc.createElement("EventManager");
+ kgameRoot.appendChild(eventManager);
+ QDomElement eventDataHandler = kgameDoc.createElement("DataHandler");
+ eventManager.appendChild(eventDataHandler);
+ QDomElement eventQueue = kgameDoc.createElement("EventQueue");
+ eventManager.appendChild(eventQueue);
+
+ QDomElement canvasEventListener = canvasDoc.createElement("EventListener");
+ canvasRoot.appendChild(canvasEventListener);
+ QDomElement canvasConditions = canvasDoc.createElement("Conditions");
+ canvasEventListener.appendChild(canvasConditions);
+#if 0
+{
+	boWarning() << k_funcinfo << "Adding a dummy condition for testing" << endl;
+	QDomElement cond = canvasDoc.createElement("Condition");
+	canvasConditions.appendChild(cond);
+
+	QDomElement events = canvasDoc.createElement("Events");
+	QDomElement statuses = canvasDoc.createElement("StatusConditions");
+	cond.appendChild(events);
+	cond.appendChild(statuses);
+
+	QDomElement matching = canvasDoc.createElement("EventMatching");
+	QDomElement event = canvasDoc.createElement("Event");
+	matching.setAttribute("IsLeft", QString::number(1));
+	matching.setAttribute("IgnoreUnitId", true);
+	matching.setAttribute("IgnorePlayerId", true);
+	matching.setAttribute("IgnoreData1", true);
+	matching.setAttribute("IgnoreData2", true);
+	event.setAttribute("Name", "UnitWithTypeDestroyed");
+	event.setAttribute("Id", 0);
+	event.setAttribute("UnitId", 0);
+	event.setAttribute("DelayedDelivery", 0);
+	event.setAttribute("HasLocation", 0);
+	event.setAttribute("Location.x", 0.0);
+	event.setAttribute("Location.y", 0.0);
+	event.setAttribute("Location.z", 0.0);
+	event.setAttribute("Data1", "");
+	event.setAttribute("Data2", "");
+	matching.appendChild(event);
+	events.appendChild(matching);
+
+	QDomElement action = canvasDoc.createElement("Action");
+	action.setAttribute("Type", "Event");
+	cond.appendChild(action);
+	QDomElement actionEvent = event.cloneNode().toElement();
+	actionEvent.setAttribute("Name", "CustomStringEvent");
+	actionEvent.setAttribute("Data1", "Foobar");
+	action.appendChild(actionEvent);
+}
+#endif
+
+ QDomElement effects = canvasDoc.createElement("Effects");
+ canvasRoot.appendChild(effects);
+
+ if (files["map/water.xml"].size() == 0) {
+	boDebug() << k_funcinfo << "Adding dummy water.xml file" << endl;
+	// Old file format - add dummy water.xml file
+	if(!addDummyWaterXML_From_0_10_To_0_10_80(files["map/water.xml"]))
+	{
+		boError() << k_funcinfo << "Couldn't add dummy water.xml file" << endl;
+		return false;
+	}
+ }
+
+ files.insert("kgame.xml", kgameDoc.toString().utf8());
+ files.insert("canvas.xml", canvasDoc.toString().utf8());
+ files.insert("players.xml", playersDoc.toString().utf8());
+ return true;
+}
+
+bool BosonFileConverter::addDummyWaterXML_From_0_10_To_0_10_80(QByteArray& waterXML)
+{
+ QDomDocument doc(QString::fromLatin1("Water"));
+ QDomElement root = doc.createElement(QString::fromLatin1("Water"));
+ doc.appendChild(root);
+ waterXML = doc.toCString();
+ return true;
+}
+
+bool BosonFileConverter::convertPlayField_From_0_10_80_To_0_10_81(QMap<QString, QByteArray>& files)
+{
+ QDomDocument kgameDoc(QString::fromLatin1("Boson"));
+ QDomDocument canvasDoc(QString::fromLatin1("Canvas"));
+ QDomDocument playersDoc(QString::fromLatin1("Players"));
+ if (!loadXMLDoc(&kgameDoc, files["kgame.xml"])) {
+	boError() << k_funcinfo << "could not load kgame.xml" << endl;
+	return false;
+ }
+ if (!loadXMLDoc(&canvasDoc, files["canvas.xml"])) {
+	boError() << k_funcinfo << "could not load canvas.xml" << endl;
+	return false;
+ }
+ QDomElement kgameRoot = kgameDoc.documentElement();
+ QDomElement canvasRoot = canvasDoc.documentElement();
+
+ // 0.10.81 development version
+#define BO_VERSION BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x05)
+ kgameRoot.setAttribute("Version", BO_VERSION);
+#undef BO_VERSION
+
+ QDomNodeList itemsList = canvasRoot.elementsByTagName("Items");
+ for (unsigned int i = 0; i < itemsList.count(); i++) {
+	QDomElement e = itemsList.item(i).toElement();
+	if (e.isNull()) {
+		boError() << k_funcinfo << "invalid Items tag" << endl;
+		return false;
+	}
+
+	// BO_TILE_SIZE is no more
+	QDomNodeList items = e.elementsByTagName("Item");
+	for (unsigned int j = 0; j < items.count(); j++) {
+		QDomElement item = items.item(j).toElement();
+		if (item.isNull()) {
+			boError() << k_funcinfo << "invalid Item tag" << endl;
+			return false;
+		}
+		bool ok;
+		float x = (float)item.attribute("x").toInt(&ok);
+		if (!ok) {
+			boError() << k_funcinfo << "x attribute is not a valid integer" << endl;
+			return false;
+		}
+		float y = (float)item.attribute("y").toInt(&ok);
+		if (!ok) {
+				boError() << k_funcinfo << "y attribute is not a valid integer" << endl;
+				return false;
+		}
+		x /= 48.0f;
+		y /= 48.0f;
+
+#warning this will be changed! -> we need to use fixed point values
+		item.setAttribute("x", x);
+		item.setAttribute("y", y);
+	}
+ }
+ files.insert("kgame.xml", kgameDoc.toString().utf8());
+ files.insert("canvas.xml", canvasDoc.toString().utf8());
+ return true;
+}
+
+bool BosonFileConverter::convertPlayField_From_0_10_81_To_0_10_82(QMap<QString, QByteArray>& files)
+{
+ QDomDocument kgameDoc(QString::fromLatin1("Boson"));
+ QDomDocument canvasDoc(QString::fromLatin1("Canvas"));
+ if (!loadXMLDoc(&kgameDoc, files["kgame.xml"])) {
+	boError() << k_funcinfo << "could not load kgame.xml" << endl;
+	return false;
+ }
+ if (!loadXMLDoc(&canvasDoc, files["canvas.xml"])) {
+	boError() << k_funcinfo << "could not load canvas.xml" << endl;
+	return false;
+ }
+ QDomElement kgameRoot = kgameDoc.documentElement();
+ QDomElement canvasRoot = canvasDoc.documentElement();
+ kgameRoot.setAttribute("Version", BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x06));
+
+ for (QDomNode node = canvasRoot.firstChild(); !node.isNull(); node = node.nextSibling()) {
+	QDomElement items = node.toElement();
+	if (items.isNull()) {
+		continue;
+	}
+	if (items.tagName() != "Items") {
+		continue;
+	}
+
+	// prior to boson 0.11 the KGameProperty IDs of the units were
+	// broken. they must be unique among different versions, however they
+	// did depend (partially) on the number of properties in the Unit.
+	// this is being fixed here now.
+	for (QDomNode itemNode = items.firstChild(); !itemNode.isNull(); itemNode = itemNode.nextSibling()) {
+		QDomElement item = itemNode.toElement();
+		if (item.isNull()) {
+			continue;
+		}
+		if (item.tagName() != "Item") {
+			continue;
+		}
+		QDomElement dataHandler = item.namedItem("DataHandler").toElement();
+		if (dataHandler.isNull()) {
+			continue;
+		}
+		for (QDomNode n = dataHandler.firstChild(); !n.isNull(); n = n.nextSibling()) {
+			QDomElement e = n.toElement();
+			if (e.isNull()) {
+				continue;
+			}
+			if (e.tagName() != "KGameProperty") {
+				continue;
+			}
+			bool ok = false;
+			unsigned int id = e.attribute("Id").toUInt(&ok);
+			if (!ok) {
+				boError() << k_funcinfo << "Id attribute in KGameProperty tag is not a valid number" << endl;
+				return false;
+			}
+			if (id < 256) {
+				// KGame property. no need to fix it.
+				continue;
+			}
+			if (id < 272) {
+				// UnitBase property. UnitBase now uses IDs from 512 to
+				// 1023.
+				id -= 256;
+				id += 512;
+			} else if (id < 582) {
+				// Unit, UnitPlugins or derived classes property.
+				if (id <= 282) {
+					// Unit property
+					// Unit now uses IDs from 1024 to 1279
+					id -= 272;
+					id += 1024;
+				} else if (id < 572) {
+					// properties in MobileUnit or Facility
+					// (only IdConstructionStep == 372 here atm)
+					// we use IDs from 1280 to 1535
+					id -= 272;
+					id += 1280;
+				} else {
+					// UnitPlugins or derived classes.
+					// IDs from 1536 to 4095 may be used here
+					id -= 572;
+					id += 1536;
+				}
+			} else {
+				boError() << k_funcinfo << "unexpected KGameProperty Id " << id << endl;
+				return false;
+			}
+			e.setAttribute("Id", id);
+		}
+	}
+
+ }
+ files.insert("kgame.xml", kgameDoc.toString().utf8());
+ files.insert("canvas.xml", canvasDoc.toString().utf8());
+ return true;
+}
+
+bool BosonFileConverter::convertPlayField_From_0_10_82_To_0_10_83(QMap<QString, QByteArray>& files)
+{
+ QDomDocument kgameDoc(QString::fromLatin1("Boson"));
+ if (!loadXMLDoc(&kgameDoc, files["kgame.xml"])) {
+	boError() << k_funcinfo << "could not load kgame.xml" << endl;
+	return false;
+ }
+ QDomElement kgameRoot = kgameDoc.documentElement();
+
+ kgameRoot.setAttribute("Version", BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x07));
+
+ unsigned int players = 0;
+ {
+	QDomDocument playersDoc(QString::fromLatin1("Players"));
+	if (!loadXMLDoc(&playersDoc, files["players.xml"])) {
+		boError() << k_funcinfo << "could not load players.xml" << endl;
+		return false;
+	}
+	QDomElement playersRoot = playersDoc.documentElement();
+	for (QDomNode node = playersRoot.firstChild(); !node.isNull(); node = node.nextSibling()) {
+		QDomElement items = node.toElement();
+		if (items.isNull()) {
+			continue;
+		}
+		if (items.tagName() != "Player") {
+			continue;
+		}
+		players++;
+	}
+ }
+
+ // Default scripts
+ QByteArray gamePy;
+ addGamePyScript_From_0_10_82_To_0_10_83(gamePy);
+ files.insert("scripts/eventlistener/game.py", gamePy);
+ QByteArray localplayerPy;
+ addLocalPlayerPyScript_From_0_10_82_To_0_10_83(localplayerPy);
+ files.insert("scripts/eventlistener/localplayer.py", localplayerPy);
+ for (unsigned int i = 0; i < players; i++) {
+	QByteArray aiPy;
+	addAIPyScript_From_0_10_82_To_0_10_83(aiPy, i);
+	files.insert(QString("scripts/eventlistener/ai-player_%1.py").arg(i), aiPy);
+ }
+
+ files.insert("kgame.xml", kgameDoc.toString().utf8());
+ return true;
+}
+
+bool BosonFileConverter::convertPlayField_From_0_10_83_To_0_10_84(QMap<QString, QByteArray>& files)
+{
+ QDomDocument kgameDoc(QString::fromLatin1("Boson"));
+ QDomDocument canvasDoc(QString::fromLatin1("Canvas"));
+ QDomDocument playersDoc(QString::fromLatin1("Players"));
+ if (!loadXMLDoc(&kgameDoc, files["kgame.xml"])) {
+	boError() << k_funcinfo << "could not load kgame.xml" << endl;
+	return false;
+ }
+ if (!loadXMLDoc(&canvasDoc, files["canvas.xml"])) {
+	boError() << k_funcinfo << "could not load canvas.xml" << endl;
+	return false;
+ }
+ if (!loadXMLDoc(&playersDoc, files["players.xml"])) {
+	boError() << k_funcinfo << "could not load players.xml" << endl;
+	return false;
+ }
+ QDomElement kgameRoot = kgameDoc.documentElement();
+ QDomElement canvasRoot = canvasDoc.documentElement();
+ QDomElement playersRoot = playersDoc.documentElement();
+
+ kgameRoot.setAttribute("Version", BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x08));
+
+ // Find out which species (eithe human or neutral) players have
+ QDomNodeList playerList = playersRoot.elementsByTagName("Player");
+ bool isHuman[playerList.count()];
+ for (unsigned int i = 0; i < playerList.count(); i++) {
+	QDomElement e = playerList.item(i).toElement();
+	int id = e.attribute("PlayerId").toInt();
+	int isneutral = e.attribute("IsNeutral").toInt();
+	isHuman[id] = !(isneutral);
+ }
+
+ // Unit type <-> health  map
+ // Human species
+ QMap<int, int> humanHealths;
+ humanHealths.insert(18, 200);
+ humanHealths.insert( 4, 250);
+ humanHealths.insert( 5, 800);
+ humanHealths.insert(14, 400);
+ humanHealths.insert( 1, 400);
+ humanHealths.insert(13, 400);
+ humanHealths.insert( 8, 400);
+ humanHealths.insert( 2, 300);
+ humanHealths.insert(11, 200);
+ humanHealths.insert( 9, 350);
+ humanHealths.insert( 6, 150);
+ humanHealths.insert(12, 500);
+ humanHealths.insert(10, 150);
+ humanHealths.insert( 3, 600);
+ humanHealths.insert(10011, 150);
+ humanHealths.insert(10006, 130);
+ humanHealths.insert(10034, 200);
+ humanHealths.insert(10008, 200);
+ humanHealths.insert(10003, 150);
+ humanHealths.insert(10002, 150);
+ humanHealths.insert(10001,  80);
+ humanHealths.insert(10009, 230);
+ humanHealths.insert(10005, 200);
+ humanHealths.insert(10000, 250);
+ humanHealths.insert(10010, 200);
+ humanHealths.insert(10004, 180);
+ humanHealths.insert(10007, 300);
+ humanHealths.insert(10035, 100);
+ humanHealths.insert(10018, 100);
+ // Neutral species
+ QMap<int, int> neutralHealths;
+ neutralHealths.insert( 1, 100);
+ neutralHealths.insert(26,  20);
+ neutralHealths.insert(25,  20);
+ neutralHealths.insert( 2, 100);
+ neutralHealths.insert(36,  60);
+ neutralHealths.insert(35,  35);
+ neutralHealths.insert(38,  60);
+ neutralHealths.insert(37,  35);
+ neutralHealths.insert(40,  60);
+ neutralHealths.insert(39,  35);
+ neutralHealths.insert( 3, 100);
+ neutralHealths.insert(45,  50);
+ neutralHealths.insert(46,  70);
+ neutralHealths.insert(12, 400);
+ neutralHealths.insert(11, 150);
+ neutralHealths.insert( 4, 100);
+ neutralHealths.insert(56,  30);
+ neutralHealths.insert(57,  15);
+ neutralHealths.insert(55,  20);
+ neutralHealths.insert(30,  30);
+ neutralHealths.insert( 5, 100);
+ neutralHealths.insert( 6, 100);
+ neutralHealths.insert( 7, 100);
+ neutralHealths.insert(16,  70);
+ neutralHealths.insert(17,  70);
+ neutralHealths.insert(15,  70);
+ neutralHealths.insert(21, 100);
+ neutralHealths.insert(22, 100);
+ neutralHealths.insert(20, 100);
+ neutralHealths.insert(51, 120);
+ neutralHealths.insert(52,  60);
+ neutralHealths.insert(50, 100);
+
+
+ // Go through all units
+ for (QDomNode n = canvasRoot.firstChild(); !n.isNull(); n = n.nextSibling()) {
+	QDomElement e = n.toElement();
+	if (e.isNull() || e.tagName() != "Items") {
+		continue;
+	}
+	int playerid = e.attribute("PlayerId").toInt();
+	bool isPlayerHuman = isHuman[playerid];
+	// Go through all units and change their absolute health to relative one
+	for (QDomNode n2 = e.firstChild(); !n2.isNull(); n2 = n2.nextSibling()) {
+		QDomElement item = n2.toElement();
+		if (item.isNull() || item.tagName() != "Item") {
+			continue;
+		}
+		// We're interested only in units
+		int rtti = item.attribute("Rtti").toInt();
+		if (rtti >= 200) { // RTTI::isUnit()
+			int type = item.attribute("Type").toInt();
+			if (!(isPlayerHuman ? humanHealths : neutralHealths).contains(type)) {
+				boError() << k_funcinfo << "Invalid unit type " << type << " for " <<
+						(isPlayerHuman ? "human" : "neutral") << " unit with id " << item.attribute("Id") <<
+						"! Skipping." << endl;
+				// Don't return, convert other units
+				continue;
+			}
+			// Replace health of the unit
+			QDomElement dataHandler = item.namedItem(QString::fromLatin1("DataHandler")).toElement();
+			QDomNodeList properties = dataHandler.elementsByTagName(QString::fromLatin1("KGameProperty"));
+			for (unsigned int k = 0; k < properties.count(); k++) {
+				QDomElement prop = properties.item(k).toElement();
+				int propertyid = prop.attribute("Id").toInt();
+				// Health property had id 512
+				if (propertyid == 512) {
+					QDomNode healthNode = prop.firstChild();
+					int maxhealth = (isPlayerHuman ? humanHealths : neutralHealths)[type];
+					bofixed healthFactor = (bofixed)healthNode.nodeValue().toInt() / maxhealth;
+					healthNode.setNodeValue(QString::number(healthFactor));
+					prop.setAttribute("Id", QString::number(512 + 20)); // UnitBase::IdHealthFactor
+					break;
+				}
+			}
+		}
+	}
+ }
+
+ files.insert("kgame.xml", kgameDoc.toString().utf8());
+ files.insert("canvas.xml", canvasDoc.toString().utf8());
+ return true;
+}
+
+bool BosonFileConverter::convertPlayField_From_0_10_84_To_0_10_85(QMap<QString, QByteArray>& files)
+{
+ QDomDocument kgameDoc(QString::fromLatin1("Boson"));
+ QDomDocument playersDoc(QString::fromLatin1("Players"));
+ QDomDocument canvasDoc(QString::fromLatin1("Canvas"));
+ if (!loadXMLDoc(&kgameDoc, files["kgame.xml"])) {
+	boError() << k_funcinfo << "could not load kgame.xml" << endl;
+	return false;
+ }
+ if (!loadXMLDoc(&playersDoc, files["players.xml"])) {
+	boError() << k_funcinfo << "could not load players.xml" << endl;
+	return false;
+ }
+ if (!loadXMLDoc(&canvasDoc, files["canvas.xml"])) {
+	boError() << k_funcinfo << "could not load canvas.xml" << endl;
+	return false;
+ }
+ QDomElement kgameRoot = kgameDoc.documentElement();
+ QDomElement playersRoot = playersDoc.documentElement();
+ QDomElement canvasRoot = canvasDoc.documentElement();
+
+ kgameRoot.setAttribute("Version", BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x09));
+
+ for (QDomNode n = playersRoot.firstChild(); !n.isNull(); n = n.nextSibling()) {
+	QDomElement player = n.toElement();
+	if (player.isNull()) {
+		continue;
+	}
+	if (player.tagName() != "Player") {
+		continue;
+	}
+
+	QDomElement speciesThemeTag = playersDoc.createElement("SpeciesTheme");
+	player.appendChild(speciesThemeTag);
+	speciesThemeTag.setAttribute("Identifier", player.attribute("SpeciesTheme"));
+	speciesThemeTag.setAttribute("TeamColor", player.attribute("TeamColor"));
+	player.removeAttribute("Identifier");
+	player.removeAttribute("TeamColor");
+
+	QDomElement unitTypes = playersDoc.createElement("UnitTypes");
+	speciesThemeTag.appendChild(unitTypes);
+
+	// AB: all already researched upgrades will be thrown away. we do not
+	//     convert them (I think it's not worth the work)
+	QDomElement upgrades = player.namedItem("Upgrades").toElement();
+	if (upgrades.isNull()) {
+		boError() << k_funcinfo << "player has no Upgrades tag" << endl;
+		return false;
+	}
+	player.removeChild(upgrades);
+
+	upgrades = playersDoc.createElement("Upgrades");
+	player.appendChild(upgrades);
+ }
+
+ for (QDomNode n = canvasRoot.firstChild(); !n.isNull(); n = n.nextSibling()) {
+	QDomElement items = n.toElement();
+	if (items.isNull() || items.tagName() != "Items") {
+		continue;
+	}
+	for (QDomNode n2 = items.firstChild(); !n2.isNull(); n2 = n2.nextSibling()) {
+		QDomElement item = n2.toElement();
+		if (item.isNull() || item.tagName() != "Item") {
+			continue;
+		}
+		int rtti = item.attribute("Rtti").toInt();
+		if (rtti < 200) { // !RTTI::isUnit()
+			continue;
+		}
+		QDomElement upgrades = canvasDoc.createElement("Upgrades");
+		item.appendChild(upgrades);
+	}
+ }
+
+ // convert Armor, Shields and SightRange properties to factor values
+ for (QDomNode n = canvasRoot.firstChild(); !n.isNull(); n = n.nextSibling()) {
+	QDomElement items = n.toElement();
+	if (items.isNull() || items.tagName() != "Items") {
+		continue;
+	}
+	for (QDomNode n2 = items.firstChild(); !n2.isNull(); n2 = n2.nextSibling()) {
+		QDomElement item = n2.toElement();
+		if (item.isNull() || item.tagName() != "Item") {
+			continue;
+		}
+		int rtti = item.attribute("Rtti").toInt();
+		if (rtti < 200) { // !RTTI::isUnit()
+			continue;
+		}
+		QDomElement dataHandler = item.namedItem(QString::fromLatin1("DataHandler")).toElement();
+		if (dataHandler.isNull()) {
+			boError() << k_funcinfo << "no DataHandler" << endl;
+			return false;
+		}
+		for (QDomNode n3 = dataHandler.firstChild(); !n3.isNull(); n3 = n3.nextSibling()) {
+			QDomElement property = n3.toElement();
+			if (property.isNull()) {
+				continue;
+			}
+			if (property.tagName() != "KGameProperty") {
+				continue;
+			}
+			bool ok = false;
+			int id = property.attribute("Id").toInt(&ok);
+			if (!ok) {
+				boError() << k_funcinfo << "Invalid Id for KGameProperty tag" << endl;
+				return false;
+			}
+			if (id == 512 + 1) { // IdArmor
+				// IdArmor -> IdArmorFactor
+				property.setAttribute("Id", QString::number(512 + 21));
+
+				// AB: we never supported armor < maxArmor
+				QDomNode text = property.firstChild();
+				bofixed v = bofixed(1.0f);
+				text.setNodeValue(QString::number(v));
+			} else if (id == 512 + 2) { // IdShields
+				// IdShields -> IdShieldsFactor
+				property.setAttribute("Id", QString::number(512 + 22));
+
+				// AB: shields were pretty much unused.
+				//     so we can assume shields == maxShields
+				QDomNode text = property.firstChild();
+				bofixed v = bofixed(1.0f);
+				text.setNodeValue(QString::number(v));
+			} else if (id == 512 + 5) { // IdSightRange
+				// IdSightRange -> IdSightRangeFactor
+				property.setAttribute("Id", QString::number(512 + 23));
+				// AB: we never supported sightRange < maxSightRange
+				QDomNode text = property.firstChild();
+				bofixed v = bofixed(1.0f);
+				text.setNodeValue(QString::number(v));
+			}
+		}
+	}
+ }
+ files.insert("kgame.xml", kgameDoc.toString().utf8());
+ files.insert("players.xml", playersDoc.toString().utf8());
+ files.insert("canvas.xml", canvasDoc.toString().utf8());
+ return true;
+}
+
+bool BosonFileConverter::convertPlayField_From_0_10_85_To_0_11(QMap<QString, QByteArray>& files)
+{
+ QDomDocument kgameDoc(QString::fromLatin1("Boson"));
+ if (!loadXMLDoc(&kgameDoc, files["kgame.xml"])) {
+	boError() << k_funcinfo << "could not load kgame.xml" << endl;
+	return false;
+ }
+ QDomElement kgameRoot = kgameDoc.documentElement();
+
+ kgameRoot.setAttribute("Version", BOSON_SAVEGAME_FORMAT_VERSION_0_11);
+
+ if (files.contains("external.xml") && files["external.xml"].size() > 0) {
+	QDomDocument canvasDoc(QString::fromLatin1("Canvas"));
+	QDomDocument externalDoc(QString::fromLatin1("External"));
+	if (!loadXMLDoc(&canvasDoc, files["canvas.xml"])) {
+		boError() << k_funcinfo << "could not load canvas.xml" << endl;
+		return false;
+	}
+	if (!loadXMLDoc(&externalDoc, files["external.xml"])) {
+		boError() << k_funcinfo << "could not load external.xml" << endl;
+		return false;
+	}
+	QDomElement canvasRoot = canvasDoc.documentElement();
+	QDomElement externalRoot = externalDoc.documentElement();
+
+	QDomElement effects = canvasRoot.namedItem("Effects").toElement();
+	if (effects.isNull()) {
+		boError() << k_funcinfo << "no Effects element in canvas.xml" << endl;
+		return false;
+	}
+	canvasRoot.removeChild(effects);
+	externalRoot.appendChild(effects);
+	files.insert("canvas.xml", canvasDoc.toString().utf8());
+	files.insert("external.xml", externalDoc.toString().utf8());
+ }
+
+ files.insert("kgame.xml", kgameDoc.toString().utf8());
+
+ files.insert("scripts/eventlistener/gamevieweventlistener.py", QByteArray());
+
+ return true;
+}
+
 void BosonFileConverter::removePropertyIds_0_9_1(const QDomNodeList& itemsList, const QStringList& ids)
 {
  for (unsigned int i = 0; i < itemsList.count(); i++) {
@@ -661,6 +1363,53 @@ void BosonFileConverter::removePropertyIds_0_9_1(const QDomNodeList& itemsList,
  }
 }
 
+bool BosonFileConverter::addGamePyScript_From_0_10_82_To_0_10_83(QByteArray& gamePy)
+{
+  return true;
+}
+
+bool BosonFileConverter::addLocalPlayerPyScript_From_0_10_82_To_0_10_83(QByteArray& localplayerPy)
+{
+  QString script =
+    "import dayandnight\n"
+    "import wind\n"
+    "\n"
+    "player = -1\n"
+    "\n"
+    "def init(id):\n"
+    "  global player\n"
+    "  player = id\n"
+    "  dayandnight.init()\n"
+    "  wind.init()\n"
+    "\n"
+    "def setPlayerId(id):\n"
+    "  global player\n"
+    "  player = id\n"
+    "\n"
+    "def advance():\n"
+    "  dayandnight.advance()\n"
+    "  wind.advance()\n";
+  localplayerPy.duplicate(script.latin1(), script.length());
+  return true;
+}
+
+bool BosonFileConverter::addAIPyScript_From_0_10_82_To_0_10_83(QByteArray& aiPy, int playerid)
+{
+  QString script =
+    "import ai\n"
+    "\n"
+    "def init(id):\n"
+    "  ai.init(id)\n"
+    "\n"
+    "def setPlayerId(id):\n"
+    "  ai.setPlayerId(id)\n"
+    "\n"
+    "def advance():\n"
+    "  ai.advance()\n";
+  aiPy.duplicate(script.latin1(), script.length());
+  return true;
+}
+
 
 bool MapToTexMap_From_0_8_To_0_9::convert(int* groundTypes, QByteArray* newMap, QByteArray* texMap)
 {
