@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include "bosonplayfield.h"
 
+#include "../bomemory/bodummymemory.h"
 #include "boversion.h"
 #include "bosonmap.h"
 #include "bosonfileconverter.h"
@@ -33,6 +34,7 @@
 #include <qdom.h>
 #include <qdatastream.h>
 #include <qfileinfo.h>
+#include <qfile.h>
 
 #include <kstandarddirs.h>
 #include <klocale.h>
@@ -92,7 +94,6 @@ BosonPlayFieldData::BosonPlayFieldData(const QString& mapFile, BosonPlayField* f
 
 bool BosonPlayFieldData::load()
 {
- BosonProfiler profiler(BosonProfiling::LoadPlayField);
  return true;
 }
 
@@ -167,7 +168,18 @@ bool BosonPlayFieldInformation::loadInformation(BPFFile* file)
 	mapXML = file->mapXMLData();
 	playersXML = file->playersData();
  }
+ QMap<QString, QByteArray> files;
+ files.insert("map/map.xml", mapXML);
+ files.insert("players.xml", playersXML);
+ return loadInformation(files);
+}
 
+// AB: note that files must contain the files in the _current_ format. no
+// conversion happens here
+bool BosonPlayFieldInformation::loadInformation(const QMap<QString, QByteArray>& files)
+{
+ QByteArray mapXML = files["map/map.xml"];
+ QByteArray playersXML = files["players.xml"];
  if (!loadPlayersInformation(playersXML)) {
 	boError() << k_funcinfo << "unable to load players information" << endl;
 	return false;
@@ -246,7 +258,6 @@ BosonPlayField::BosonPlayField(QObject* parent) : QObject(parent, "BosonPlayFiel
  mMap = 0;
  mFile = 0;
  mPreLoaded = false;
- mLoaded = false;
  mPlayFieldInformation = new BosonPlayFieldInformation();
  mDescription = new BPFDescription();
 }
@@ -266,7 +277,7 @@ bool BosonPlayField::preLoadAllPlayFields()
  boDebug() << k_funcinfo << endl;
 
  // TODO: ensure that UI doesn't block (i.e. call process events)
- BosonProfiler profiler(BosonProfiling::PreLoadPlayFields);
+ BosonProfiler profiler("Preload all playfields");
  if (BosonData::bosonData()->availablePlayFields().count() > 0) {
 	boWarning() << k_funcinfo << "playFields already loaded" << endl;
 	return true;
@@ -336,14 +347,16 @@ boWarning() << k_funcinfo << "obsolete" << endl;
 
 QString BosonPlayField::defaultPlayField()
 {
+ QString _default = "weareunderattack.bpf";
+
  QStringList l = boData->availablePlayFields();
- if (l.contains(DEFAULT_PLAYFIELD)) {
-	return DEFAULT_PLAYFIELD;
+ if (l.contains(_default)) {
+	return _default;
  }
  if (l.count() == 0) {
 	return QString::null;
  }
- boWarning() << k_funcinfo << "cannot find " << DEFAULT_PLAYFIELD << " map - using " << l[0] << " instead" << endl;
+ boWarning() << k_funcinfo << "cannot find " << _default<< " map - using " << l[0] << " instead" << endl;
  return l[0];
 }
 
@@ -362,7 +375,8 @@ bool BosonPlayField::preLoadPlayField(const QString& file)
 	boError() << k_funcinfo << "Could not load playfield information" << endl;
 	return false;
  }
- QByteArray descriptionXML = mFile->descriptionData();
+ QMap<QString, QByteArray> descriptions = mFile->descriptionsData();
+ QByteArray descriptionXML = descriptions["C/description.xml"];
  if (descriptionXML.size() == 0 && !mFile->hasMapDirectory()) {
 	boError() << k_funcinfo << "old savegame format not supported." << endl;
 	return false;
@@ -397,6 +411,10 @@ bool BosonPlayField::loadPlayField(const QMap<QString, QByteArray>& files)
 	boError() << k_funcinfo << "no texmap found" << endl;
 	return false;
  }
+ if (!files.contains("map/water.xml")) {
+	boError() << k_funcinfo << "no water.xml found" << endl;
+	return false;
+ }
 
  // AB: C/description.xml is mandatory. all other languages are optional.
  if (!files.contains("C/description.xml")) {
@@ -404,7 +422,11 @@ bool BosonPlayField::loadPlayField(const QMap<QString, QByteArray>& files)
 	return false;
  }
 
- if (!loadMapFromFile(files["map/map.xml"], files["map/heightmap.png"], files["map/texmap"])) {
+ if (!mPlayFieldInformation->loadInformation(files)) {
+	boError() << k_funcinfo << "Could not load playfield information" << endl;
+	return false;
+ }
+ if (!loadMapFromFile(files["map/map.xml"], files["map/heightmap.png"], files["map/texmap"], files["map/water.xml"])) {
 	boError() << k_funcinfo << "error loading the map" << endl;
 	return false;
  }
@@ -433,7 +455,7 @@ bool BosonPlayField::loadDescriptionFromFile(const QByteArray& xml)
  return true;
 }
 
-bool BosonPlayField::loadMapFromFile(const QByteArray& mapXML, const QByteArray& heightMapImage, const QByteArray& texMap)
+bool BosonPlayField::loadMapFromFile(const QByteArray& mapXML, const QByteArray& heightMapImage, const QByteArray& texMap, const QByteArray& waterXML)
 {
  boDebug() << k_funcinfo << endl;
  if (mapXML.size() == 0) {
@@ -448,6 +470,10 @@ bool BosonPlayField::loadMapFromFile(const QByteArray& mapXML, const QByteArray&
 	boError() << k_funcinfo << "empty texmap array" << endl;
 	return false;
  }
+ if (waterXML.size() == 0) {
+	boError() << k_funcinfo << "empty byte array for waterXML" << endl;
+	return false;
+ }
  delete mMap;
  mMap = new BosonMap(this);
  bool ret = mMap->loadMapFromFile(mapXML);
@@ -476,9 +502,11 @@ bool BosonPlayField::loadMapFromFile(const QByteArray& mapXML, const QByteArray&
 	boError() << k_funcinfo << "Could not load map (height map failed)" << endl;
 	return false;
  }
-
- // Create color map
- mMap->createColorMap();
+ ret = mMap->loadWaterFromFile(waterXML);
+ if (!ret) {
+	boError() << k_funcinfo << "Could not load water" << endl;
+	return false;
+ }
 
  return ret;
 }
@@ -501,6 +529,15 @@ QByteArray BosonPlayField::saveMapToFile()
  return mMap->saveMapToFile();
 }
 
+QByteArray BosonPlayField::saveWaterToFile()
+{
+ if (!mMap) {
+	boError() << k_funcinfo << "NULL map" << endl;
+	return QByteArray();
+ }
+ return mMap->saveWaterToFile();
+}
+
 QByteArray BosonPlayField::saveTexMapToFile()
 {
  if (!mMap) {
@@ -551,7 +588,6 @@ void BosonPlayField::deleteMap()
 void BosonPlayField::finalizeLoading()
 {
  mPreLoaded = true;
- mLoaded = true;
 }
 
 QString BosonPlayField::playFieldName() const
@@ -661,6 +697,7 @@ bool BosonPlayField::loadFromDiskToFiles(QMap<QString, QByteArray>& destFiles)
  QByteArray heightMap = mFile->heightMapData();
  QByteArray texMap = mFile->texMapData();
  QByteArray mapXML = mFile->mapXMLData();
+ QByteArray waterXML = mFile->waterXMLData();
  QByteArray playersXML = mFile->playersData();
  QByteArray canvasXML = mFile->canvasData();
  QByteArray kgameXML = mFile->kgameData();
@@ -704,20 +741,32 @@ bool BosonPlayField::loadFromDiskToFiles(QMap<QString, QByteArray>& destFiles)
 	return false;
  }
 
- QByteArray descriptionXML = mFile->descriptionData();
- if (descriptionXML.size() == 0) {
+ QMap<QString, QByteArray> descriptions = mFile->descriptionsData();
+ if (!descriptions.contains("C/description.xml")) {
+	boError() << k_funcinfo << "no C/description.xml file found" << endl;
+	return false;
+ }
+ if (descriptions["C/description.xml"].size() == 0) {
 	boError() << k_funcinfo << "empty default description.xml" << endl;
 	return false;
  }
+ for (QMap<QString, QByteArray>::iterator it = descriptions.begin(); it != descriptions.end(); ++it) {
+	destFiles.insert(it.key(), it.data());
+ }
+
+ QMap<QString, QByteArray> scripts = mFile->scriptsData();
+ for (QMap<QString, QByteArray>::iterator it = scripts.begin(); it != scripts.end(); ++it) {
+	destFiles.insert(it.key(), it.data());
+ }
 
  QByteArray externalXML = mFile->externalData();
  destFiles.insert("map/texmap", texMap);
  destFiles.insert("map/heightmap.png", heightMap);
  destFiles.insert("map/map.xml", mapXML);
+ destFiles.insert("map/water.xml", waterXML);
  destFiles.insert("players.xml", playersXML);
  destFiles.insert("canvas.xml", canvasXML);
  destFiles.insert("kgame.xml", kgameXML);
- destFiles.insert("C/description.xml", descriptionXML);
  if (externalXML.size() != 0) {
 	// AB: externalXML is optional only. only for loading games.
 	destFiles.insert("external.xml", externalXML);
@@ -812,11 +861,17 @@ bool BosonPlayField::savePlayFieldToFiles(QMap<QString, QByteArray>& files)
  QByteArray mapXML;
  QByteArray heightMap;
  QByteArray texMap;
+ QByteArray waterXML;
  mapXML = saveMapToFile();
  if (mapXML.size() == 0) {
 	boError() << k_funcinfo << "failed saving the map" << endl;
 	return false;
  }
+ waterXML = saveWaterToFile();
+ if (waterXML.size() == 0) {
+	boError() << k_funcinfo << "failed saving water" << endl;
+	return false;
+ }
  heightMap = mMap->saveHeightMapImage();
  if (heightMap.size() == 0) {
 	boError() << k_funcinfo << "failed saving the heightmap" << endl;
@@ -837,6 +892,7 @@ bool BosonPlayField::savePlayFieldToFiles(QMap<QString, QByteArray>& files)
  }
 
  files.insert("map/map.xml", mapXML);
+ files.insert("map/water.xml", waterXML);
  files.insert("map/heightmap.png", heightMap);
  files.insert("map/texmap", texMap);
  files.insert("C/description.xml", descriptionXML);
@@ -969,6 +1025,90 @@ bool BosonPlayField::convertFilesToCurrentFormat(QMap<QString, QByteArray>& dest
 		}
 		break;
 	}
+	case BOSON_SAVEGAME_FORMAT_VERSION_0_10:
+	{
+		boDebug() << k_funcinfo << "converting from 0.10 to 0.10.80 format" << endl;
+		BosonFileConverter converter;
+		if (!converter.convertPlayField_From_0_10_To_0_10_80(destFiles)) {
+			boError() << k_funcinfo << "could not convert from boson 0.10 to boson 0.10.80 file format" << endl;
+			ret = false;
+		} else {
+			ret = true;
+		}
+		break;
+	}
+	case BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x04): // development version ("0.10.80")
+	{
+		boDebug() << k_funcinfo << "converting from 0.10.80 to 0.10.81 format" << endl;
+		BosonFileConverter converter;
+		if (!converter.convertPlayField_From_0_10_80_To_0_10_81(destFiles)) {
+			boError() << k_funcinfo << "could not convert from boson 0.10.80 to boson 0.10.81 file format" << endl;
+			ret = false;
+		} else {
+			ret = true;
+		}
+		break;
+	}
+	case BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x05): // development version ("0.10.81")
+	{
+		boDebug() << k_funcinfo << "converting from 0.10.81 to 0.10.82 format" << endl;
+		BosonFileConverter converter;
+		if (!converter.convertPlayField_From_0_10_81_To_0_10_82(destFiles)) {
+			boError() << k_funcinfo << "could not convert from boson 0.10.81 to boson 0.10.82 file format" << endl;
+			ret = false;
+		} else {
+			ret = true;
+		}
+		break;
+	}
+	case BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x06): // development version ("0.10.82")
+	{
+		boDebug() << k_funcinfo << "converting from 0.10.82 to 0.10.83 format" << endl;
+		BosonFileConverter converter;
+		if (!converter.convertPlayField_From_0_10_82_To_0_10_83(destFiles)) {
+			boError() << k_funcinfo << "could not convert from boson 0.10.82 to boson 0.10.83 file format" << endl;
+			ret = false;
+		} else {
+			ret = true;
+		}
+		break;
+	}
+	case BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x07): // development version ("0.10.83")
+	{
+		boDebug() << k_funcinfo << "converting from 0.10.83 to 0.10.84 format" << endl;
+		BosonFileConverter converter;
+		if (!converter.convertPlayField_From_0_10_83_To_0_10_84(destFiles)) {
+			boError() << k_funcinfo << "could not convert from boson 0.10.83 to boson 0.10.84 file format" << endl;
+			ret = false;
+		} else {
+			ret = true;
+		}
+		break;
+	}
+	case BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x08): // development version ("0.10.84")
+	{
+		boDebug() << k_funcinfo << "converting from 0.10.84 to 0.10.85 format" << endl;
+		BosonFileConverter converter;
+		if (!converter.convertPlayField_From_0_10_84_To_0_10_85(destFiles)) {
+			boError() << k_funcinfo << "could not convert from boson 0.10.84 to boson 0.10.85 file format" << endl;
+			ret = false;
+		} else {
+			ret = true;
+		}
+		break;
+	}
+	case BOSON_MAKE_SAVEGAME_FORMAT_VERSION(0x00, 0x02, 0x09): // development version ("0.10.85")
+	{
+		boDebug() << k_funcinfo << "converting from 0.10.85 to 0.11 format" << endl;
+		BosonFileConverter converter;
+		if (!converter.convertPlayField_From_0_10_85_To_0_11(destFiles)) {
+			boError() << k_funcinfo << "could not convert from boson 0.10.85 to boson 0.11 file format" << endl;
+			ret = false;
+		} else {
+			ret = true;
+		}
+		break;
+	}
 	default:
 		*handled = false;
 		ret = true;
