@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -148,15 +148,35 @@ public:
 	void setCanvas(BosonCanvas*);
 	void setPlayField(BosonPlayField*);
 
-	// AB: temporary function only, for the scenario-code-replacing process.
-	bool loadNewGame(const QByteArray& playersXML, const QByteArray& canvasXML);
-
-	bool saveToFile(const QString& file);
+	/**
+	 * Save the game to @p files.
+	 **/
 	bool saveToFiles(QMap<QString, QByteArray>& files);
+
+	/**
+	 * Just like @ref saveToFiles, but this takes care of converting the
+	 * saved data into the correct format for a playfield file. This data
+	 * can be used to start <em>new</em> games, as opposed to loading saved
+	 * games.
+	 **/
 	bool savePlayFieldToFiles(QMap<QString, QByteArray>& files);
+
+	/**
+	 * Save the data from @p files to the @p file. Use @ref saveToFiles or
+	 * @ref savePlayFieldToFiles to actually save the data.
+	 **/
 	static bool saveToFile(const QMap<QString, QByteArray>& files, const QString& file);
 
-	LoadingStatus loadingStatus() const;
+	/**
+	 * Start a game from a @p files QMap. The @p files should come from a
+	 * file that was saved using @ref saveToFile
+	 *
+	 * Note that the actual file and data loading all happens in @ref
+	 * BosonStarting. This just starts the actual game, i.e. makes sure that
+	 * items are added and so on. The playfield has been loaded already,
+	 * too.
+	 **/
+	bool startFromFiles(const QMap<QString, QByteArray>& files);
 
 	/**
 	 * Load the XML file in @p xml into @p doc and display an error message
@@ -191,12 +211,14 @@ protected:
 	QCString savePlayersAsXML();
 	QCString saveCanvasAsXML();
 	QCString saveExternalAsXML();
+	bool saveEventListenerScripts(QMap<QString, QByteArray>* files);
 
 	bool loadVersionFromXML(const QString&); // takes kgame.xml
 	bool loadKGameFromXML(const QString&);
 	bool loadPlayersFromXML(const QString&);
 	bool loadCanvasFromXML(const QString&);
 	bool loadExternalFromXML(const QString&);
+	bool loadEventListenerScripts(const QMap<QString, QByteArray>& files);
 
 	bool convertSaveGameToPlayField(QMap<QString, QByteArray>& files);
 
@@ -207,15 +229,27 @@ protected:
 	void addSaveError(SaveLoadError::ErrorType, const QString& text = QString::null, const QString& caption = QString::null);
 	void addLoadError(SaveLoadError::ErrorType, const QString& text = QString::null, const QString& caption = QString::null);
 
+	/**
+	 * This fixes player ids of filenames for saving.
+	 *
+	 * Other classes (e.g. the @ref BoEventManager) save their files to @p
+	 * files and sometimes use the playerid in the filenames (e.g.
+	 * ai-player_id.py). This method reads those filenames, removes them
+	 * from @p files and re-adds them with the player id replaced by the
+	 * index of the player in the game list (which is used in boson .bpf and
+	 * .bsg files).
+	 *
+	 * Only files with a "-player_id" substring, where id is a number, are
+	 * touched. That substring is replaced by "-player_n", where n is the
+	 * index of the player.
+	 *
+	 * @return FALSE on error, otherwise TRUE.
+	 **/
+	bool convertPlayerIdsToIndices(QMap<QString, QByteArray>& files) const;
+
 private:
 	void initBoson();
 
-	/**
-	 * KGame::systemAddPlayer is protected, this is used as a workaround.
-	 * Not nice, but working.
-	 **/
-	void systemAddPlayer(KPlayer*);
-
 private:
 	BosonSaveLoadPrivate* d;
 };
