@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,15 +31,11 @@
 
 class KSimpleConfig;
 class SpeciesTheme;
-class BosonEffect;
-class BosonEffectProperties;
 class Unit;
-class BoAction;
 class QString;
 template<class T> class QIntDict;
 template<class T1, class T2> class QMap;
 
-
 /**
  * @short This class holds properties for @ref BosonWeapon
  *
@@ -67,36 +63,59 @@ class BosonWeaponProperties : public PluginProperties
      **/
     inline bool canShootAtLandUnits() const  { return mCanShootAtLandUnits; }
     /**
-     * @return The weapon range of this unit. It's a number of cells, so multiply
-     *  with BO_TILE_SIZE to use it on the canvas.
-    **/
-    inline unsigned long int range() const  { return mRange; }
+     * @return The weapon range of this unit. It's a number of cells
+     **/
+    inline unsigned long int range() const  { return mRange.value(upgradesCollection()); }
+    /**
+     * @return Maximum flying distance of the missile of this weapon.
+     * If the missile hasn't caught the target after flying this distance, the
+     *  missile runs out of fuel and self-detonates.
+     * This has effect only for missile weapons.
+     **/
+    inline bofixed maxFlyDistance() const  { return mMaxFlyDistance; }
+    /**
+     * @return Starting angle (in degrees) of the missile of this weapon.
+     * This is the vertical angle that the missile will get when it's launched.
+     *  If it's 0, missile will start to fly horizontally towards the target,
+     *  if it's 90, missile will first fly upward and then turn to it's target.
+     *  If it's -1, then missile will start flying directly towards it's
+     *  target.
+     * Must be in range 0-90 or -1.
+     * This has effect only for missile weapons.
+     **/
+    inline bofixed startAngle() const  { return mStartAngle; }
     /**
      * @return The number of advance calls until the weapon is reloaded
      **/
-    inline unsigned int reloadingTime() const  { return mReload; }
+    inline unsigned int reloadingTime() const  { return mReloadingTime.value(upgradesCollection()); }
     /**
      * The damage this unit makes to other units. Negative values means
      * repairing
      **/
-    inline long int damage() const  { return mDamage; }
+    inline long int damage() const  { return mDamage.value(upgradesCollection()); }
     /**
      * @return Damage range of missile of this unit, e.g. range in what units will be damaged
      **/
-    inline float damageRange() const  { return mDamageRange; }
+    inline bofixed damageRange() const  { return mDamageRange.value(upgradesCollection()); }
     /**
      * @return Full damage range of missile of this unit, e.g. range in what
      *  units will be damaged by damage value (farther they'll be damaged less)
      **/
-    inline float fullDamageRange() const  { return mFullDamageRange; }
+    inline bofixed fullDamageRange() const  { return mFullDamageRange.value(upgradesCollection()); }
     /**
      * @return Maximum speed that missile of this weapon can have or 0 if speed is infinite
      **/
-    inline float speed() const  { return mSpeed; }
+    inline bofixed speed() const  { return mSpeed.value(upgradesCollection()); }
     /**
      * @return Acceleration speed of missile of this unit
      **/
-    inline float accelerationSpeed() const  { return mAccelerationSpeed; }
+    inline bofixed accelerationSpeed() const  { return mAccelerationSpeed; }
+    /**
+     * @return tangens of turning speed of the missile of this weapon.
+     * Turning speed specifies how fast the missile can turn.
+     * It has effect only for missile weapons.
+     **/
+    bofixed turningSpeed() const  { return mTurningSpeed; }
     /**
      * @return Filename of 3d model of shot of this weapon.
      * Only used in unit editor
@@ -111,7 +130,7 @@ class BosonWeaponProperties : public PluginProperties
      * Offset is relative to the center point of unit and is used when creating
      * new shot.
      **/
-    inline BoVector3 offset() const  { return mOffset; }
+    inline BoVector3Fixed offset() const  { return mOffset; }
 
     inline BosonShot::Type shotType() const  { return mShotType; }
     /**
@@ -119,9 +138,7 @@ class BosonWeaponProperties : public PluginProperties
      * Note that this is height per cell, it should be multiplied by distance of
      * shot to get final height.
      **/
-    inline float height() const  { return mHeight; }
-
-    inline BosonModel* model() const  { return mModel; }
+    inline bofixed height() const  { return mHeight; }
 
     /**
      * @return First part of the sound filename - e.g. "shoot" if the file
@@ -130,8 +147,6 @@ class BosonWeaponProperties : public PluginProperties
      **/
     QString sound(int soundEvent) const;
 
-    void playSound(WeaponSoundEvent event) const;
-
     QMap<int, QString> sounds() const;
 
     unsigned long int id() const  { return mId; }
@@ -143,85 +158,99 @@ class BosonWeaponProperties : public PluginProperties
      **/
     bool autoUse() const  { return mAutoUse; }
 
+    bool takeTargetVeloIntoAccount() const  { return mTakeTargetVeloIntoAccount; }
+
     /**
      * Creates new shot
      * @param attacker Unit that fired this shot
+     * @param weapon The weapon that fired this shot
      * @param pos Position (center point) of attacker. Note that offset is added to this value
      * @param target Position of shot's target point
      * @return Created shot. Note that it's not added to canvas.
      * Note that pos or target may not be used depending on shot's type
      **/
-    BosonShot* newShot(Unit* attacker, BoVector3 pos, BoVector3 target) const;
-
-    QPtrList<BosonEffect> newShootEffects(BoVector3 pos, float rotation) const;
-    QPtrList<BosonEffect> newFlyEffects(BoVector3 pos, float rotation) const;
-    QPtrList<BosonEffect> newHitEffects(BoVector3 pos) const;
+    BosonShot* newShot(Unit* attacker, const BosonWeapon* weapon, BoVector3Fixed pos, BoVector3Fixed target) const;
+    BosonShot* newShot(Unit* attacker, const BosonWeapon* weapon, BoVector3Fixed pos, Unit* target) const;
 
     QValueList<unsigned long int> shootEffectIds() const  { return mShootEffectIds; }
     QValueList<unsigned long int> flyEffectIds() const  { return mFlyEffectIds; }
     QValueList<unsigned long int> hitEffectIds() const  { return mHitEffectIds; }
 
     virtual QString name() const;
-    virtual void loadPlugin(KSimpleConfig* config)  { loadPlugin(config, true); }
-    virtual void loadPlugin(KSimpleConfig* config, bool full = true);
+    virtual void loadPlugin(KSimpleConfig* config);
     virtual void savePlugin(KSimpleConfig* config);
     virtual int pluginType() const  { return Weapon; }
 
-    QIntDict<BoAction>* actions()  { return &mActions; }
+    const QMap<int, QString>* actionStrings() const  { return &mActionStrings; }
+
+
 
   protected:
     void setWeaponName(QString str)  { mName = str; }
-    void setDamage(long int damage)  { mDamage = damage; }
-    void setDamageRange(float range)  { mDamageRange = range; }
-    void setFullDamageRange(float range)  { mFullDamageRange = range; }
-    void setReloadingTime(unsigned int reload)  { mReload = reload; }
-    void setRange(unsigned long int range)  { mRange = range; }
     void setCanShootAtAirUnits(bool can)  { mCanShootAtAirUnits = can; }
     void setCanShootAtLandUnits(bool can)  { mCanShootAtLandUnits = can; }
-    void setSpeed(float speed)  { mSpeed = speed; }
-    void setAccelerationSpeed(float speed)  { mAccelerationSpeed = speed; }
+    void setAccelerationSpeed(bofixed speed)  { mAccelerationSpeed = speed; }
     void setModelFileName(QString file)  { mModelFileName = file; }
     void setShootEffectIds(QValueList<unsigned long int> ids)  { mShootEffectIds = ids; }
     void setFlyEffectIds(QValueList<unsigned long int> ids)  { mFlyEffectIds = ids; }
     void setHitEffectIds(QValueList<unsigned long int> ids)  { mHitEffectIds = ids; }
-    void setOffset(BoVector3 o)  { mOffset = o; }
-    void setHeight(float height)  { mHeight = height; }
+    void setOffset(BoVector3Fixed o)  { mOffset = o; }
+    void setHeight(bofixed height)  { mHeight = height; }
     void setSound(int event, QString filename);
     void setAutoUse(bool use)  { mAutoUse = use; }
+    void setTakeTargetVeloIntoAccount(bool take)  { mTakeTargetVeloIntoAccount = take; }
+    void setMaxFlyDistance(bofixed dist)  { mMaxFlyDistance = dist; }
+    void setTurningSpeed(bofixed s)  { mTurningSpeed = s; }
+    void setStartAngle(bofixed a)  { mStartAngle = a; }
 
     void reset();
     void loadAction(UnitAction type, KSimpleConfig* cfg, const QString& key, bool useDefault = false);
 
+    /**
+     * Calls @ref insertULongWeaponBaseValue. @p name is converted to be a valid
+     * weapon name, i.e. to "Weapon_ID:name", where ID is @ref id - 1.
+     **/
+    bool insertULongWeaponBaseValue(unsigned long int v, const QString& name, const QString& type = "MaxValue");
+    bool insertLongWeaponBaseValue(long int v, const QString& name, const QString& type = "MaxValue");
+    bool insertBoFixedWeaponBaseValue(bofixed v, const QString& name, const QString& type = "MaxValue");
+
+    /**
+     * @return @ref ulongBaseValue, but with @p name modified so, that it is a
+     * valid weapon property name (see also @ref insertULongWeaponBaseValue).
+     **/
+    unsigned long int ulongWeaponBaseValue(const QString& name, const QString& type = "MaxValue", unsigned long int defaultValue = 0) const;
+    long int longWeaponBaseValue(const QString& name, const QString& type = "MaxValue", long int defaultValue = 0) const;
+    bofixed bofixedWeaponBaseValue(const QString& name, const QString& type = "MaxValue", bofixed defaultValue = 0) const;
+
     friend class BoUnitEditor;
-    friend class UpgradeProperties;
 
   private:
+
     unsigned long int mId;
-    unsigned long int mRange;
-    long int mDamage;
-    float mDamageRange;
-    float mFullDamageRange;
+    BoUpgradeableProperty<unsigned long int> mRange;
+    BoUpgradeableProperty<long int> mDamage;
+    BoUpgradeableProperty<bofixed> mDamageRange;
+    BoUpgradeableProperty<bofixed> mFullDamageRange;
+    BoUpgradeableProperty<unsigned long int> mReloadingTime;
+    BoUpgradeableProperty<bofixed> mSpeed;
     bool mCanShootAtAirUnits;
     bool mCanShootAtLandUnits;
-    unsigned int mReload;
-    float mSpeed;
-    float mAccelerationSpeed;
-    float mHeight;
+    bofixed mAccelerationSpeed;
+    bofixed mHeight;
     BosonShot::Type mShotType;
-    BosonModel* mModel;
     QString mModelFileName;
     QString mName;
-    QPtrList<BosonEffectProperties> mShootEffects;
-    QPtrList<BosonEffectProperties> mFlyEffects;
-    QPtrList<BosonEffectProperties> mHitEffects;
-    // FIXME: these are only needed in editor mode. In normal mode, they only waste memory
     QValueList<unsigned long int> mShootEffectIds;
     QValueList<unsigned long int> mFlyEffectIds;
     QValueList<unsigned long int> mHitEffectIds;
-    BoVector3 mOffset;
+    BoVector3Fixed mOffset;
     QMap<int, QString> mSounds;
-    QIntDict<BoAction> mActions;
+    QMap<int, QString> mActionStrings;
     bool mAutoUse;
+    bool mTakeTargetVeloIntoAccount;
+    bofixed mMaxFlyDistance;
+    bofixed mTurningSpeed;
+    bofixed mStartAngle;
 };
 
 
@@ -259,10 +288,21 @@ class BosonWeapon : public UnitPlugin
     /**
      * Reload the weapon.
      **/
-    virtual void advance(unsigned int advanceCount)
+    virtual void advance(unsigned int advanceCallsCount)
     {
-      Q_UNUSED(advanceCount);
-      reload();
+      Q_UNUSED(advanceCallsCount);
+    }
+
+    inline void reload(unsigned int reloadBy)
+    {
+      if(mReloadCounter > (int)reloadBy)
+      {
+        mReloadCounter = mReloadCounter - reloadBy;
+      }
+      else
+      {
+        mReloadCounter = 0;
+      }
     }
 
     virtual bool saveAsXML(QDomElement& root) const;
@@ -280,7 +320,7 @@ class BosonWeapon : public UnitPlugin
     inline const BosonWeaponProperties* properties() const  { return mProp; }
 
     void shoot(Unit* u);
-    void shoot(const BoVector3& target);
+    void shoot(const BoVector3Fixed& target);
 
     /**
      * Lay mine at current location of unit
@@ -293,21 +333,56 @@ class BosonWeapon : public UnitPlugin
      * Drop bomb from the current location of unit
      * If bomb is dropped, returns true. If weapon is not a bomb, not reloaded
      * or some other error occurs, returns false
+     * @param hvelocity initial horizontal velocity of the bomb
      **/
-    bool dropBomb();
+    bool dropBomb(const BoVector2Fixed& hvelocity);
 
+    virtual void unitDestroyed(Unit*) {}
     virtual void itemRemoved(BosonItem*) {}
 
-  protected:
-    inline void reload()  { if(mReloadCounter > 0) { mReloadCounter = mReloadCounter - 1; } }
+    /**
+     * @return The weapon range of this unit. It's a number of cells
+     **/
+    inline unsigned long int range() const  { return mRange.value(upgradesCollection()); }
+    /**
+     * @return The number of advance calls until the weapon is reloaded
+     **/
+    inline unsigned int reloadingTime() const  { return mReloadingTime.value(upgradesCollection()); }
+    /**
+     * The damage this unit makes to other units. Negative values means
+     * repairing
+     **/
+    inline long int damage() const  { return mDamage.value(upgradesCollection()); }
+    /**
+     * @return Damage range of missile of this unit, e.g. range in what units will be damaged
+     **/
+    inline bofixed damageRange() const  { return mDamageRange.value(upgradesCollection()); }
+    /**
+     * @return Full damage range of missile of this unit, e.g. range in what
+     *  units will be damaged by damage value (farther they'll be damaged less)
+     **/
+    inline bofixed fullDamageRange() const  { return mFullDamageRange.value(upgradesCollection()); }
+    /**
+     * @return Maximum speed that missile of this weapon can have or 0 if speed is infinite
+     **/
+    inline bofixed speed() const  { return mSpeed.value(upgradesCollection()); }
 
-    void shoot(const BoVector3& pos, const BoVector3& target);
+
+  protected:
+    void shoot(const BoVector3Fixed& pos, const BoVector3Fixed& target);
 
     void registerWeaponData(int weaponNumber, KGamePropertyBase* prop, int id, bool local = true);
 
   private:
     const BosonWeaponProperties* mProp;
     KGameProperty<int> mReloadCounter;
+
+    BoUpgradeableProperty<unsigned long int> mRange;
+    BoUpgradeableProperty<long int> mDamage;
+    BoUpgradeableProperty<bofixed> mDamageRange;
+    BoUpgradeableProperty<bofixed> mFullDamageRange;
+    BoUpgradeableProperty<unsigned long int> mReloadingTime;
+    BoUpgradeableProperty<bofixed> mSpeed;
 };
 
 #endif // BOSONWEAPON_H
