@@ -1,6 +1,7 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2002-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,25 +17,30 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
-#ifndef BOSONSPRITE_H
-#define BOSONSPRITE_H
+#ifndef BOSONITEM_H
+#define BOSONITEM_H
 
 #include "../defines.h"
-
-#include <GL/gl.h>
+#include "../bomath.h"
+#include <bogl.h>
 
 #include <qglobal.h>
 
 class BosonCanvas;
 class BosonCollisions;
-class SelectBox;
 class BosonModel;
 class BosonAnimation;
-class BoFrame;
-class QRect;
 class Cell;
-class BosonParticleSystem;
 class BosonItemPropertyHandler;
+class Player;
+class SpeciesTheme;
+template<class T> class BoVector2;
+template<class T> class BoVector3;
+template<class T> class BoRect;
+typedef BoRect<bofixed> BoRectFixed;
+typedef BoVector2<bofixed> BoVector2Fixed;
+typedef BoVector3<bofixed> BoVector3Fixed;
+class BoFrustum;
 
 class KGamePropertyHandler;
 class KGamePropertyBase;
@@ -143,16 +149,63 @@ private:
 class BosonItem : public BosonItemProperties
 {
 public:
+	enum PropertyIds {
+		// BosonItem used IDs from 256 to 511.
+		// all values above 511 are reserved for derived classes, up to
+		// 28671 (28672 and greater might be used by KGame again for
+		// automatic IDs)
+	};
+
 	/**
 	 * Note: when you subclass this class you must set the width/height in
 	 * order to make correct use of it! See @ref setSize
 	 **/
-	BosonItem(BosonModel* model, BosonCanvas*);
+	BosonItem(Player* owner, BosonCanvas*);
 	virtual ~BosonItem();
 
+	/**
+	 * @return A string that identifies the model that should be used for
+	 * this item. This model will get assigned to this model after
+	 * construction using @ref BosonItemRenderer::setModel.
+	 **/
+	virtual QString getModelIdForItem() const = 0;
+
+	/**
+	 * Update the current animation mode using @ref getAnimationMode
+	 **/
+	void updateAnimationMode();
+
+	/**
+	 * @return The current animation mode, see also @ref
+	 * updateAnimationMode.
+	 **/
+	int animationMode() const
+	{
+		return mAnimationMode;
+	}
+
+	/**
+	 * Called after the constructor. You can do all kinds of initializations
+	 * here - try to use this instead of the constructor whenever possible.
+	 *
+	 * This is in particular useful when you need to ensure that a certain
+	 * condition applies, e.g. that canvas() is non-NULL. You can simply
+	 * return FALSE in that case and the item will be deleted.
+	 *
+	 * @return TRUE on success or FALSE on failure (the item will be deleted
+	 * then)
+	 **/
+	virtual bool init() { return true; }
+
 	inline BosonCanvas* canvas() const { return mCanvas; }
-	inline BosonModel* model() const { return mModel; }
 	BosonCollisions* collisions() const;
+	inline Player* owner() const { return mOwner; }
+
+	/**
+	 * @return owner()->speciesTheme()
+	 **/
+	SpeciesTheme* speciesTheme() const;
+
 
 	/**
 	 * @return Unique identifier of this object type. E.g. RTTI::Unit for
@@ -174,10 +227,10 @@ public:
 
 	/**
 	 * @return The x-coordinate of the left edge of the object.Note that
-	 * this has <em>nothing</em> to do with the OpenGL coordinates. 
+	 * this has <em>nothing</em> to do with the OpenGL coordinates.
 	 * These are the internal canvas coordinates.
 	 **/
-	inline float x() const { return mX; }
+	inline bofixed x() const { return mX; }
 
 	/**
 	 * @return The y-coordinate of the unit on the canvas. Note that this is
@@ -185,45 +238,50 @@ public:
 	 * @ref x() and @ref y() (acutally canvas and OpenGL coordiantes are
 	 * equal for z)
 	 **/
-	inline float y() const { return mY; }
+	inline bofixed y() const { return mY; }
 
 	/**
 	 * @return The z-position of the item.
 	 **/
-	inline float z() const { return mZ; }
+	inline bofixed z() const { return mZ; }
 
 	/**
-	 * @param width Width in cells * BO_TILE_SIZE. FIXME: should be changed to cell value (float!!)
-	 * @param height Height in cells * BO_TILE_SIZE. FIXME: should be changed to cell value (float!!)
+	 * @param width Width in cells
+	 * @param height Height in cells
 	 **/
-	void setSize(int width, int height, float depth);
+	void setSize(bofixed width, bofixed height, bofixed depth);
 
 	// note: for GLunit all frames must have the same width/height!
 	// different depth is ok!
-	inline int width() const { return mWidth; }
-	inline int height() const { return mHeight; }
+	inline bofixed width() const { return mWidth; }
+	inline bofixed height() const { return mHeight; }
 	/**
 	 * @return item's height in z-direction.
 	 * This does not depend on any OpenGL stuff (model, frame etc) and should be
 	 * same on all clients, so it can be used for collision detection and
 	 * pathfinding
 	 **/
-	inline float depth() const { return mDepth; }
+	inline bofixed depth() const { return mDepth; }
 
-	inline float leftEdge() const { return x(); }
-	inline float topEdge() const { return y(); }
-	inline float rightEdge() const { return (leftEdge() + width() - 1); }
-	inline float bottomEdge() const { return (topEdge() + height() - 1); }
+	inline bofixed leftEdge() const { return x(); }
+	inline bofixed topEdge() const { return y(); }
+	inline bofixed rightEdge() const { return leftEdge() + width(); }
+	inline bofixed bottomEdge() const { return topEdge() + height(); }
 
-	QRect boundingRect() const;
-	QRect boundingRectAdvanced() const;
+	inline bofixed centerX() const { return x() + width() / 2; };
+	inline bofixed centerY() const { return y() + height() / 2; };
+	inline bofixed centerZ() const { return z() + depth() / 2; };
+	BoVector2Fixed center() const;
+
+	BoRectFixed boundingRect() const;
+	BoRectFixed boundingRectAdvanced() const;
 
 	/**
 	 * Move the item to @p nx, @p ny, @p nz. Note that it is moved without
 	 * parameter checking, i.e. we don't check whether these cooridnates are
 	 * valid.
 	 **/
-	inline void move(float nx, float ny, float nz)
+	inline void move(bofixed nx, bofixed ny, bofixed nz)
 	{
 		moveBy(nx - x(), ny - y(), nz - z());
 	}
@@ -233,7 +291,7 @@ public:
 	 * is done!
 	 *
 	 * Also note that when you move an item by e.g. (1,1,0) that this new
-	 * position is <em>not</em> guaranteed to an actual position on the 
+	 * position is <em>not</em> guaranteed to an actual position on the
 	 * ground. I mean when the item (e.g. a unit) is on grass and you move
 	 * it by a certain amount so that it moves to another cell, then it is
 	 * possible that that cell is at a different height than previous cell.
@@ -245,13 +303,13 @@ public:
 	 * checking) you should reimplement this method and call it in your
 	 * implementation.
 	 **/
-	virtual void moveBy(float dx, float dy, float dz)
+	virtual void moveBy(bofixed dx, bofixed dy, bofixed dz)
 	{
 		if (dx || dy || dz) {
 			removeFromCells();
 			setPos(x() + dx, y() + dy, z() + dz);
 			addToCells();
-			moveParticleSystems(x(), y(), z());
+			setEffectsPositionDirty(true);
 		}
 	}
 
@@ -260,26 +318,14 @@ public:
 	 * @return TRUE if the object is selected, i.e. a select box should be
 	 * drawn around it. Otherwise FALSE.
 	 **/
-	bool isSelected() const { return mSelectBox != 0; }
-
-	/**
-	 * Render the item. This assumes the modelview matrix was already
-	 * translated and rotated to the correct position.
-	 * @param lod See @ref BoFrame::renderFrame
-	 **/
-	void renderItem(unsigned int lod = 0);
-
-	inline GLuint displayList() const
+	bool isSelected(bool* isGroupLeader = 0) const
 	{
-		return mDisplayList;
+		if (isGroupLeader) {
+			*isGroupLeader = mIsGroupLeaderOfSelection;
+		}
+		return mIsSelected;
 	}
 
-	/**
-	 * @return The select box of this item, or NULL if it is not selected.
-	 * See @ref isSelected.
-	 **/
-	inline SelectBox* selectBox() const { return mSelectBox; }
-
 	/**
 	 * @return The (cell-)coordinates of the left-top cell this object
 	 * occupies. This can be used for very efficient collision detection.
@@ -295,10 +341,10 @@ public:
 	 * instance. You can use it to find out which cells a unit would occupy
 	 * if it was at a certain position.
 	 **/
-	inline static void leftTopCell(int* left, int* top, float leftEdge, float topEdge)
+	inline static void leftTopCell(int* left, int* top, bofixed leftEdge, bofixed topEdge)
 	{
-		*left = (int)(leftEdge / BO_TILE_SIZE);
-		*top = (int)(topEdge / BO_TILE_SIZE);
+		*left = (int)(leftEdge);
+		*top = (int)(topEdge);
 	}
 
 	/**
@@ -316,10 +362,10 @@ public:
 	 * instance. You can use it to find out which cells a unit would occupy
 	 * if it was at a certain position.
 	 **/
-	inline static void rightBottomCell(int* right, int* bottom, float rightEdge, float bottomEdge)
+	inline static void rightBottomCell(int* right, int* bottom, bofixed rightEdge, bofixed bottomEdge)
 	{
-		*right = (int)(rightEdge / BO_TILE_SIZE);
-		*bottom= (int)(bottomEdge / BO_TILE_SIZE);
+		*right = (int)(rightEdge);
+		*bottom= (int)(bottomEdge);
 	}
 
 	/**
@@ -343,14 +389,24 @@ public:
 	 * calculate which cells the unit would occupy if it was at a certain
 	 * position.
 	 **/
-	static void makeCells(Cell* allCells, QPtrVector<Cell>* cells, int left, int right, int top, int bottom, int mapWidth, int mapHeight);
+	static void makeCells(Cell* allCells, QPtrVector<Cell>* cells, const BoRect<bofixed>& rect, int mapWidth, int mapHeight);
 
+	/**
+	 * @return Whether this unit collides with given unit.
+	 * It uses width, height and depth of the unit for accurate collision
+	 * detection.
+	 **/
 	bool bosonCollidesWith(BosonItem* item) const;
 
-	inline float xVelocity() const { return mXVelocity; }
-	inline float yVelocity() const { return mYVelocity; }
-	inline float zVelocity() const { return mZVelocity; }
-	void setVelocity(float vx, float vy, float vz = 0.0)
+	/**
+	 * Same as above, but uses box with given coords instead of actual item.
+	 **/
+	bool bosonCollidesWith(const BoVector3Fixed& v1, const BoVector3Fixed& v2) const;
+
+	inline bofixed xVelocity() const { return mXVelocity; }
+	inline bofixed yVelocity() const { return mYVelocity; }
+	inline bofixed zVelocity() const { return mZVelocity; }
+	void setVelocity(bofixed vx, bofixed vy, bofixed vz = 0)
 	{
 		mXVelocity = vx;
 		mYVelocity = vy;
@@ -360,13 +416,13 @@ public:
 	/**
 	 * @return Current speed of this item
 	 **/
-	inline float speed() const { return mCurrentSpeed; }
-	inline void setSpeed(float s) { mCurrentSpeed = s; }
+	inline bofixed speed() const { return mCurrentSpeed; }
+	inline void setSpeed(bofixed s) { mCurrentSpeed = s; }
 	/**
 	 * @return Maximum speed this item may have
 	 **/
-	inline float maxSpeed() const { return mMaxSpeed; }
-	inline void setMaxSpeed(float maxspeed) { mMaxSpeed = maxspeed; }
+	inline bofixed maxSpeed() const { return mMaxSpeed; }
+	inline void setMaxSpeed(bofixed maxspeed) { mMaxSpeed = maxspeed; }
 	/**
 	 * Raises speed by @ref accelerationSpeed unless @ref currentSpeed is
 	 * @ref maxSpeed
@@ -375,75 +431,56 @@ public:
 	/**
 	 * Lowers speed by @ref decelerationSpeed unless @ref currentSpeed is 0
 	 **/
-	inline void decelerate() { mCurrentSpeed = QMAX(0, speed() - decelerationSpeed()); }
+	inline void decelerate() { mCurrentSpeed = QMAX(bofixed(0), speed() - decelerationSpeed()); }
 	/**
 	 * @return How fast this unit accelerates.
 	 * Acceleration speed shows how much speed of unit changes per advance call.
 	 **/
-	inline float accelerationSpeed() const { return mAccelerationSpeed; }
-	inline void setAccelerationSpeed(float s) { mAccelerationSpeed = s; }
+	inline bofixed accelerationSpeed() const { return mAccelerationSpeed; }
+	inline void setAccelerationSpeed(bofixed s) { mAccelerationSpeed = s; }
 	/**
 	 * @return How fast this unit decelerates.
 	 * Deceleration speed shows how much speed of unit changes per advance call.
 	 **/
-	inline float decelerationSpeed() const { return mDecelerationSpeed; }
-	inline void setDecelerationSpeed(float s) { mDecelerationSpeed = s; }
+	inline bofixed decelerationSpeed() const { return mDecelerationSpeed; }
+	inline void setDecelerationSpeed(bofixed s) { mDecelerationSpeed = s; }
 	/**
 	 * @return How much this unit moves before stopping completely
 	 * This is distance that item will move before it completely stops when it
 	 * starts deceleration now and continues it until stopping
 	 **/
-	inline float decelerationDistance() const { return (mCurrentSpeed / mDecelerationSpeed) / 2 * mCurrentSpeed; }
+	inline bofixed decelerationDistance() const { return (mCurrentSpeed / mDecelerationSpeed) / 2 * mCurrentSpeed; }
 
 
 	inline void setVisible(bool v) { mIsVisible = v; }
 	inline bool isVisible() const { return mIsVisible; }
 
 	/**
-	 * Set the animation mode. Only possible if the construction of the unit
-	 * is completed (i.e. the construction step is greater or equal to @ref
-	 * glConstructionSteps).
+	 * Reloads everything that can be reloaded. For example in @ref Unit
+	 * this does weapon reloading and shield reloading currently.
 	 *
-	 * See @ref BosonModel::animation for more information about @p mode.
-	 **/
-	void setAnimationMode(int mode);
-
-
-	/**
-	 * Make this item animated, i.e. call @ref advance on it. You don't need
-	 * to set it animated if you don't reimplement @ref advance. See also
-	 * @ref BosonCanvas::addAnimation and @ref BosonCanvas::slotAdvance
-	 **/
-	void setAnimated(bool a);
-
-	/**
-	 * Increase the animation timer and once it exceeds the @ref
-	 * BoAnimation::speed set a new frame.
-	 **/
-	void animate();
-
-	/**
-	 * See @ref Unit::advance
+	 * This could also do self-reperature, if the item provides such
+	 * features.
 	 *
-	 * This advance implementation manages the animation of the item. Call
-	 * it in your implementation, if you want to provide animations!
+	 * This method is not called every advance call, but in a certain
+	 * interval only. The parameter @p count specifies how much reloading
+	 * should be done. For example if this is called every 5 advance calls,
+	 * then @p count will probably be 5.
 	 **/
-	inline virtual void advance(unsigned int )
+	virtual void reload(unsigned int count)
 	{
-		if (mCurrentAnimation) {
-			animate();
-		}
+		Q_UNUSED(count);
 	}
 
 	/**
 	 * See @ref Unit::advanceFunction
 	 **/
-	inline virtual void advanceFunction(unsigned int /*advanceCount*/) { }
+	inline virtual void advanceFunction(unsigned int /*advanceCallCount*/) { }
 
 	/**
 	 * See @ref Unit::advanceFunction2
 	 **/
-	inline virtual void advanceFunction2(unsigned int /*advanceCount*/) { }
+	inline virtual void advanceFunction2(unsigned int /*advanceCallCount*/) { }
 
 	/**
 	 * Used to synchronize the advance function for the next advance call.
@@ -461,67 +498,31 @@ public:
 	inline virtual void syncAdvanceFunction2() { }
 
 	/**
-	 * @return The factor you need to multiply BO_GL_CELL_SIZE with to
-	 * achieve the depth (height in z-direction) of the unit. Note that this
-	 * value <em>must not</em> be used in pathfinding or so, but only in
-	 * OpenGL!
+	 * Called when @p item is removed from the canvas (right before the item
+	 * is deleted).
+	 * If this item stores any references to @p item, it should remove them
+	 * NOW.
+	 *
+	 * The default implementation does nothing
 	 **/
-	inline float glDepthMultiplier() const { return mGLDepthMultiplier; }
-
-	/**
-	 * See @ref glDepthMultiplier. Note that the depth of the unit
-	 * (i.e. the height in z-direction) is allowed to change for
-	 * different frames, but the depth multiplier not!
-	 **/
-	void setGLDepthMultiplier(float d);
-
-	void setGLConstructionStep(unsigned int step);
-
-	/**
-	 * Convenience method. Note that the glConstructionSteps differ from the
-	 * @ref Facility::constructionSteps completely!
-	 * @return model()->constructionSteps()
-	 **/
-	unsigned int glConstructionSteps() const;
-
-	/**
-	 * For OpenGL performance <em>only</em>! Do <em>not</em> use outside
-	 * OpenGL! Especially not in pathfinding!
-	 * @ return The radius of the bounding sphere. See @ref
-	 * BosonBigDisplayBase::sphereInFrustum
-	 **/
-	inline float boundingSphereRadius() const { return mBoundingSphereRadius; }
-
-	void setBoundingSphereRadius(float r) { mBoundingSphereRadius = r; }
+	virtual void itemRemoved(BosonItem* item) { Q_UNUSED(item); }
 
 	/**
 	 * @return unit's current rotation around z-axis. This is used for rotating
-	 * unit to correct direction when moving. Note that this value
-	 * <em>mustn't</em> be used for pathfinding and similar actions, as of
-	 * floating point operations!
+	 * unit to correct direction when moving.
 	 **/
-	inline float rotation() const { return mRotation; }
-	void setRotation(float r) { rotateParticleSystems(r - mRotation, 0.0, 0.0, 1.0); mRotation = r; }
+	inline bofixed rotation() const { return mRotation; }
+	void setRotation(bofixed r) { mRotation = r; setEffectsRotationDirty(true); }
 
-	inline float xRotation() const { return mXRotation; }
-	void setXRotation(float r) { rotateParticleSystems(r - mXRotation, 1.0, 0.0, 0.0); mXRotation = r; }
+	inline bofixed xRotation() const { return mXRotation; }
+	void setXRotation(bofixed r) { mXRotation = r; setEffectsRotationDirty(true); }
 
-	inline float yRotation() const { return mYRotation; }
-	void setYRotation(float r) { rotateParticleSystems(r - mYRotation, 0.0, 1.0, 0.0); mYRotation = r; }
+	inline bofixed yRotation() const { return mYRotation; }
+	void setYRotation(bofixed r) { mYRotation = r; setEffectsRotationDirty(true); }
 
 
-	void moveParticleSystems(float x, float y, float z);
-	void rotateParticleSystems(float angle, float x, float y, float z);
-
-	virtual QPtrList<BosonParticleSystem>* particleSystems() const;
-
-// TODO: add something like virtual bool canBeSelected() const = 0; or so! some
-// objects (like missiles) just can't be selected usually.
 	/**
 	 * Select this unit, i.e. construct the select box - see @ref isSelected.
-	 * Note that @ref SelectBox is constructed very fast - the box is
-	 * constructed only once and the display list is stored as a static
-	 * variable.
 	 * @param markAsLeader The leader of a group of units/items will have a
 	 * slightly different select box (e.g. another color or so).
 	 **/
@@ -535,7 +536,6 @@ public:
 	virtual bool saveAsXML(QDomElement&);
 	virtual bool loadFromXML(const QDomElement&);
 
-protected:
 	/**
 	 * @return The team color this item should get rendered with. This
 	 * should be the @ref Player::teamColor of the owner, if applicable. For
@@ -544,18 +544,35 @@ protected:
 	 **/
 	virtual const QColor* teamColor() const = 0;
 
-private:
+	inline void setEffectsPositionDirty(bool d)
+	{
+		mEffectsPositionIsDirty = d;
+	}
+	bool isEffectsPositionDirty() const
+	{
+		return mEffectsPositionIsDirty;
+	}
+
+	inline void setEffectsRotationDirty(bool d)
+	{
+		mEffectsRotationIsDirty = d;
+	}
+	bool isEffectsRotationDirty() const
+	{
+		return mEffectsRotationIsDirty;
+	}
+
+protected:
 	/**
-	 * Change the currently displayed frame. Note that you can't set the
-	 * construction frames here, as they are generated on the fly and don't
-	 * reside as an actual frame in the .3ds file.
-	 *
-	 * You usually don't want to call this, but rather @ref setAnimationMode
-	 * instead.
+	 * @return The current animation mode. @ref UnitAnimationIdle by
+	 * default. Note that currently only units use animations, but they
+	 * should be supported for every item. See also @ref
+	 * updateAnimationMode.
 	 **/
-	void setFrame(int _frame);
-	inline int frame() const { return mFrame; }
-	unsigned int frameCount() const;
+	virtual int getAnimationMode() const
+	{
+		return UnitAnimationIdle;
+	}
 
 private:
 	/**
@@ -564,7 +581,7 @@ private:
 	 *
 	 * You should call @ref addToCells after this function.
 	 **/
-	inline void setPos(float x, float y, float z)
+	inline void setPos(bofixed x, bofixed y, bofixed z)
 	{
 		mX = x;
 		mY = y;
@@ -572,18 +589,6 @@ private:
 		mCellsDirty = true;
 	}
 
-	void setCurrentFrame(BoFrame* frame);
-
-	/**
-	 * Cache the display list from the current frame. Don't use this
-	 * directly - use @ref setCurrentFrame (or rather @ref setFrame) instead.
-	 **/
-	inline void setDisplayList(GLuint l)
-	{
-		mDisplayList = l;
-	}
-
-
 	/**
 	 * Add the item to the cells on the canvas. This should get called
 	 * whenever the item has been moved in any way (i.e. also when its size
@@ -601,53 +606,43 @@ private:
 
 private:
 	BosonCanvas* mCanvas;
-	BosonModel* mModel;
-	BosonAnimation* mCurrentAnimation;
+	Player* mOwner;
 	// FIXME: use KGameProperty here. We can do so, since we don't use
 	// QCanvasSprite anymore.
 	unsigned long int mId;
-	float mX;
-	float mY;
-	float mZ;
-	int mWidth;
-	int mHeight;
-	float mDepth;
+	bofixed mX;
+	bofixed mY;
+	bofixed mZ;
+	bofixed mWidth;
+	bofixed mHeight;
+	bofixed mDepth;
 
-	float mXVelocity;
-	float mYVelocity;
-	float mZVelocity;
+	bofixed mXVelocity;
+	bofixed mYVelocity;
+	bofixed mZVelocity;
 
-	float mCurrentSpeed;
-	float mMaxSpeed;
-	float mAccelerationSpeed;
-	float mDecelerationSpeed;
+	bofixed mCurrentSpeed;
+	bofixed mMaxSpeed;
+	bofixed mAccelerationSpeed;
+	bofixed mDecelerationSpeed;
 
-// OpenGL values. should not be used for pathfinding and so on. Most stoff
-// shouldn't be stored in save() either
-	float mRotation;
-	float mXRotation;
-	float mYRotation;
-	float mGLDepthMultiplier;
-	BoFrame* mCurrentFrame;
-	GLuint mDisplayList;
-	unsigned int mGLConstructionStep;
-	unsigned int mFrame;
-	unsigned int mAnimationCounter;
+	bofixed mRotation;
+	bofixed mXRotation;
+	bofixed mYRotation;
 
-	// these are for OpenGL performance only. no need to store in save() and
-	// don't use for anything except OpenGL!
-//	float mCenterX;
-//	float mCenterY;
-//	float mCenterZ;
-	float mBoundingSphereRadius;
-
-
-	bool mIsAnimated;
-	SelectBox* mSelectBox;
+	bool mIsSelected;
+	bool mIsGroupLeaderOfSelection;
 
 	QPtrVector<Cell>* mCells;
 	bool mCellsDirty;
 	bool mIsVisible;
+
+	bool mEffectsPositionIsDirty;
+	bool mEffectsRotationIsDirty;
+
+	// AB: this is NOT saved to any file! it is calculated on the fly by
+	// updateAnimationMode() and getAnimationMode()
+	int mAnimationMode;
 };
 
 #endif
