@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2004 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,11 +25,12 @@
 #include "bosonitem.h"
 #include <qptrlist.h>
 
+#include <kgame/kgameproperty.h>
+
 class Unit;
 class Player;
-class BosonParticleSystem;
-class BosonParticleSystemProperties;
 class BosonWeaponProperties;
+class BosonWeapon;
 class UnitProperties;
 
 class QDomElement;
@@ -47,17 +48,32 @@ template<class T> class QPtrList;
 class BosonShot : public BosonItem
 {
   public:
+    enum PropertyIds
+    {
+        // BosonShot uses IDs from 4096 to 4159
+        // derived classes may use 4160 to 8191.
+        // WARNING: because we need to add every Id with a name to the
+        // (static!!) property map, IDs must be unique among different classes.
+        // Therefore we must not use the same ID twice, even if that's in
+        // different classes.
+        IdWeaponDamage = 4096,
+        IdWeaponDamageRange = 4097,
+        IdWeaponFullDamageRange = 4098,
+        IdWeaponSpeed = 4099
+    };
+
     /**
      * Type of the shot
      **/
     enum Type
     {
       Bullet = 0,
-      Missile,
+      Rocket,
       Explosion,
       Mine,
       Bomb,
-      Fragment
+      Fragment,
+      Missile
     };
 
     /**
@@ -68,13 +84,18 @@ class BosonShot : public BosonItem
      **/
     BosonShot(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
 
-    /**
-     * Same as above, except that it meant to be used when you don't have weapon
-     * properties, but still want to use a model
-     **/
-    BosonShot(Player* owner, BosonCanvas* canvas, BosonModel* model);
-
     BosonShot(Player* owner, BosonCanvas* canvas);
+    ~BosonShot();
+
+    virtual QString getModelIdForItem() const;
+    virtual bool init();
+
+    /**
+     * Apply the values from @p weapon to this shot. @p weapon is the weapon
+     * shooting this shot, its values (like @ref damage) are derived to this
+     * object.
+     **/
+    virtual void applyWeapon(const BosonWeapon* weapon);
 
     /**
      * @return Weapon properties of this shot if it has one.
@@ -94,13 +115,13 @@ class BosonShot : public BosonItem
      * If you supply weapon properties in ctor, this equals to properties()->damageRange().
      * Otherwise, you should re-implement it in derived classes
      **/
-    virtual float damageRange() const;
+    virtual bofixed damageRange() const;
     /**
      * @return Full damage range of this shot.
      * If you supply weapon properties in ctor, this equals to properties()->fullDamageRange().
      * Otherwise, you should re-implement it in derived classes
      **/
-    virtual float fullDamageRange() const;
+    virtual bofixed fullDamageRange() const;
 
     /**
      * @return If this shot is active.
@@ -143,10 +164,13 @@ class BosonShot : public BosonItem
 
     virtual void explode();
 
+  private:
+    static void initStatic();
+
   protected:
     virtual const QColor* teamColor() const;
 
-    inline void setActive(bool a)  { mActive = a; }
+    void setInactive();
     inline void setProperties(const BosonWeaponProperties* p)  { mProp = p; }
 
     virtual void advanceMoveInternal() {}
@@ -154,11 +178,19 @@ class BosonShot : public BosonItem
 
     virtual void moveToTarget() {}
 
-  private:
-    void init();
-
+  protected:
     bool mActive;
     const BosonWeaponProperties* mProp;
+
+    // values from BosonWeapon. note that some classes may not use them at all
+    // (e.g. reimplement damage() with their own values)
+    KGameProperty<long int> mWeaponDamage;
+    KGameProperty<bofixed> mWeaponDamageRange;
+    KGameProperty<bofixed> mWeaponFullDamageRange;
+    KGameProperty<bofixed> mWeaponSpeed;
+
+  private:
+    void initPrivate();
 };
 
 
@@ -172,64 +204,131 @@ class BosonShot : public BosonItem
 class BosonShotBullet : public BosonShot
 {
   public:
-    BosonShotBullet(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 target);
+    enum PropertyIds
+    {
+      // 4160 to 4223 (4160+63) allowed here
+    };
+
     BosonShotBullet(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
+    virtual bool init();
 
     // Bullets won't be saved becaused they will immediately "explode" and
     //  become inactive
     virtual bool saveAsXML(QDomElement&) { return true; }
     virtual bool loadFromXML(const QDomElement&)
     {
-      setActive(false);
+      setInactive();
       return true;
     }
 
+    void setTarget(const BoVector3Fixed& target);
+
     inline virtual int type() const { return BosonShot::Bullet; }
 
   protected:
     virtual void moveToTarget();
 
   private:
-    BoVector3 mTarget;
+    static void initStatic();
+
+  private:
+    BoVector3Fixed mTarget;
 };
 
 
 /**
- * @short Shot class for missiles
+ * @short Shot class for rockets
  *
- * Missile is a shot that flies through the air from initial position to target
+ * Rockets is a shot that flies through the air from initial position to target.
+ *
+ * Note that rockets are not guided.
+ *
+ * @author Rivo Laks <rivolaks@hot.ee>
+ **/
+class BosonShotRocket : public BosonShot
+{
+  public:
+    enum PropertyIds
+    {
+      // 4224 to 4287 (4224+63) allowed here
+      IdTotalDist = 4224 + 0,
+      IdPassedDist = 4224 + 1,
+      IdZ = 4224 + 2,
+      IdEffectVelocity = 4224 + 3,
+      IdMaxHeight = 4224 + 4
+    };
+
+    BosonShotRocket(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
+    ~BosonShotRocket();
+
+    virtual bool saveAsXML(QDomElement& root);
+    virtual bool loadFromXML(const QDomElement& root);
+
+    void init(const BoVector3Fixed& pos, const BoVector3Fixed& target);
+
+    inline virtual int type() const { return BosonShot::Rocket; }
+
+  protected:
+    virtual void advanceMoveInternal();
+
+    virtual void moveToTarget();
+
+  private:
+    static void initStatic();
+
+  private:
+    BoVector3Fixed mVelo;
+    BoVector3Fixed mTarget;
+    KGameProperty<bofixed> mTotalDist;
+    KGameProperty<bofixed> mPassedDist;
+    KGameProperty<bofixed> mZ;
+    KGameProperty<bofixed> mMaxHeight;
+    bofixed mEffectVelo;
+    bofixed mLastDistToTarget;
+};
+
+
+/**
+ * @short Shot class for missile
+ *
+ * Missile is a shot that flies trough the air, trying to reach it's target.
+ *
+ * Missiles are guided, that is, they will follow their targets.
  *
  * @author Rivo Laks <rivolaks@hot.ee>
  **/
 class BosonShotMissile : public BosonShot
 {
   public:
-    BosonShotMissile(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 pos, BoVector3 target);
+    enum PropertyIds
+    {
+      // 4544 to 4607 (4544+63) allowed here
+      IdPassedDist = 4544 + 0,
+    };
+
     BosonShotMissile(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
     ~BosonShotMissile();
 
     virtual bool saveAsXML(QDomElement& root);
     virtual bool loadFromXML(const QDomElement& root);
 
-    virtual const QPtrList<BosonParticleSystem>* particleSystems() const  { return mFlyParticleSystems; }
+    void init(const BoVector3Fixed& pos, Unit* target);
 
-    inline virtual int type() const { return BosonShot::Missile; }
+    inline virtual int type() const { return BosonShot::Rocket; }
 
   protected:
-    void setParticleSystems(const QPtrList<BosonParticleSystem>& list);
     virtual void advanceMoveInternal();
 
     virtual void moveToTarget();
 
   private:
-    BoVector3 mVelo;
-    BoVector3 mTarget;
-    float mTotalDist;
-    float mPassedDist;
-    float mZ;
-    float mParticleVelo;
-    float mMaxHeight;
-    QPtrList<BosonParticleSystem>* mFlyParticleSystems;
+    static void initStatic();
+
+  private:
+    Unit* mTarget;
+    KGameProperty<bofixed> mPassedDist;
+    // This is _normalized_ velocity, i.e. just direction
+    BoVector3Fixed mVelo;
 };
 
 
@@ -245,10 +344,18 @@ class BosonShotMissile : public BosonShot
 class BosonShotExplosion : public BosonShot
 {
   public:
-    BosonShotExplosion(Player* owner, BosonCanvas* canvas, BoVector3 pos, long int damage, float damagerange, float fulldamagerange, int delay);
+    enum PropertyIds
+    {
+      // 4288 to 4351 (4288+63) allowed here
+      IdDamage = 4288 + 0,
+      IdDamageRange = 4288 + 1,
+      IdFullDamageRange  = 4288 + 2,
+      IdDelay = 4288 + 3
+    };
+
     BosonShotExplosion(Player* owner, BosonCanvas* canvas);
 
-    void activate(const BoVector3& pos, long int damange, float damageRange, float fulldamagerange, int delay);
+    void activate(const BoVector3Fixed& pos, long int damange, bofixed damageRange, bofixed fulldamagerange, int delay);
 
     virtual bool saveAsXML(QDomElement& root);
     virtual bool loadFromXML(const QDomElement& root);
@@ -256,17 +363,20 @@ class BosonShotExplosion : public BosonShot
     inline virtual int type() const { return BosonShot::Explosion; }
 
     virtual long int damage() const  { return mDamage; }
-    virtual float damageRange() const  { return mDamageRange; }
-    virtual float fullDamageRange() const  { return mFullDamageRange; }
+    virtual bofixed damageRange() const  { return mDamageRange; }
+    virtual bofixed fullDamageRange() const  { return mFullDamageRange; }
 
   protected:
     virtual void advanceMoveInternal();
 
   private:
-    long int mDamage;
-    float mDamageRange;
-    float mFullDamageRange;
-    int mDelay;
+    static void initStatic();
+
+  private:
+    KGameProperty<long int> mDamage;
+    KGameProperty<bofixed> mDamageRange;
+    KGameProperty<bofixed> mFullDamageRange;
+    KGameProperty<int> mDelay;
 };
 
 
@@ -281,19 +391,29 @@ class BosonShotExplosion : public BosonShot
 class BosonShotMine : public BosonShot
 {
   public:
-    BosonShotMine(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 pos);
+    enum PropertyIds
+    {
+      // 4352 to 4415 (4352+63) allowed here
+      IdActivated = 4352 + 0,
+    };
+
     BosonShotMine(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
 
     virtual bool saveAsXML(QDomElement& root);
     virtual bool loadFromXML(const QDomElement& root);
 
+    void init(const BoVector3Fixed& pos);
+
     inline virtual int type() const { return BosonShot::Mine; }
 
   protected:
     virtual void advanceMoveInternal();
 
   private:
-    bool mActivated;
+    static void initStatic();
+
+  private:
+    KGamePropertyBool mActivated;
 };
 
 
@@ -308,19 +428,32 @@ class BosonShotMine : public BosonShot
 class BosonShotBomb : public BosonShot
 {
   public:
-    BosonShotBomb(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop, BoVector3 pos);
+    enum PropertyIds
+    {
+      // 4416 to 4479 (4416+63) allowed here
+      IdActivated = 4416 + 0,
+    };
+
     BosonShotBomb(Player* owner, BosonCanvas* canvas, const BosonWeaponProperties* prop);
 
     virtual bool saveAsXML(QDomElement& root);
     virtual bool loadFromXML(const QDomElement& root);
 
+    void init(const BoVector3Fixed& pos);
+
+    void setHorizontalVelocity(const BoVector2Fixed& velo) { mHorizontalVelocity = velo; }
+
     inline virtual int type() const { return BosonShot::Bomb; }
 
   protected:
     virtual void advanceMoveInternal();
 
   private:
-    bool mActivated;
+    static void initStatic();
+
+  private:
+    KGamePropertyBool mActivated;
+    BoVector2Fixed mHorizontalVelocity;
 };
 
 
@@ -334,33 +467,40 @@ class BosonShotBomb : public BosonShot
 class BosonShotFragment : public BosonShot
 {
   public:
-    BosonShotFragment(Player* owner, BosonCanvas* canvas, BosonModel* model, BoVector3 pos,
-        const UnitProperties* unitproperties);
-    BosonShotFragment(Player* owner, BosonCanvas* canvas, BosonModel* model);
+    enum PropertyIds
+    {
+      // 4480 to 4543 (4480+63) allowed here
+    };
+
+    BosonShotFragment(Player* owner, BosonCanvas* canvas);
     ~BosonShotFragment();
 
-    void activate(const BoVector3& pos, const UnitProperties* unitProperties);
+    virtual QString getModelIdForItem() const;
+
+    void activate(const BoVector3Fixed& pos, const UnitProperties* unitProperties);
 
     virtual bool saveAsXML(QDomElement& root);
     virtual bool loadFromXML(const QDomElement& root);
 
     virtual long int damage() const;
-    virtual float damageRange() const;
-    virtual float fullDamageRange() const;
+    virtual bofixed damageRange() const;
+    virtual bofixed fullDamageRange() const;
 
     inline virtual int type() const { return BosonShot::Fragment; }
 
-    virtual const QPtrList<BosonParticleSystem>* particleSystems() const  { return mParticleSystems; }
-
-    virtual void explode();
+    const UnitProperties* unitProperties() const
+    {
+      return mUnitProperties;
+    }
 
   protected:
-    void setParticleSystems(const QPtrList<BosonParticleSystem>& list);
     virtual void advanceMoveInternal();
 
   private:
-    QPtrList<BosonParticleSystem>* mParticleSystems;
-    BoVector3 mVelo;
+    static void initStatic();
+
+  private:
+    BoVector3Fixed mVelo;
     const UnitProperties* mUnitProperties;
 };
 
