@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2003 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include "kgamemodeldebug.h"
 #include "kgamemodeldebug.moc"
 
+#include "../bomemory/bodummymemory.h"
 #include "bo3dtools.h"
 #include "bomatrixwidget.h"
 #include "bodebug.h"
@@ -42,6 +43,7 @@
 #include <qpushbutton.h>
 #include <qbuttongroup.h>
 #include <qdir.h>
+#include <qptrqueue.h>
 
 #include <ksimpleconfig.h>
 #include <klistbox.h>
@@ -148,15 +150,15 @@ public:
 	class TrackKeyDataLin3 : public TrackKeyData {
 	public:
 		TrackKeyDataLin3() : TrackKeyData() { }
-		BoVector3 value;
-		BoVector3 dd;
-		BoVector3 ds;
+		BoVector3Float value;
+		BoVector3Float dd;
+		BoVector3Float ds;
 	};
 
 	class TrackKeyDataQuat : public TrackKeyData {
 	public:
 		TrackKeyDataQuat() : TrackKeyData() { }
-		BoVector3 axis;
+		BoVector3Float axis;
 		float angle;
 		BoQuaternion q;
 		BoQuaternion dd;
@@ -371,9 +373,9 @@ public:
 	}
 	virtual int type() const { return (int)Bo3DSTrackKey::TrackLin3; }
 
-	BoVector3 eval(float t) const
+	BoVector3Float eval(float t) const
 	{
-		return BoVector3();
+		return BoVector3Float();
 	}
 
 protected:
@@ -541,9 +543,9 @@ protected:
 		key->mTCB.copyTCB(_key->tcb);
 		key->quatData()->axis.set(_key->axis);
 		key->quatData()->angle = _key->angle;
-		key->quatData()->q.set(_key->q[3], BoVector3(_key->q));
-		key->quatData()->dd.set(_key->dd[3], BoVector3(_key->dd));
-		key->quatData()->ds.set(_key->ds[3], BoVector3(_key->ds));
+		key->quatData()->q.set(_key->q[3], BoVector3Float(_key->q));
+		key->quatData()->dd.set(_key->dd[3], BoVector3Float(_key->dd));
+		key->quatData()->ds.set(_key->ds[3], BoVector3Float(_key->ds));
 	}
 };
 
@@ -879,9 +881,9 @@ void BoTrackWidget::configureKey(QListViewItem* item, Bo3DSTrackKey* key, int ty
 	}
 	case Bo3DSTrackKey::TrackLin3:
 	{
-		item->setText(mKeyData0, key->lin3Data()->value.debugString(prec));
-		item->setText(mKeyData1, key->lin3Data()->dd.debugString(prec));
-		item->setText(mKeyData2, key->lin3Data()->ds.debugString(prec));
+		item->setText(mKeyData0, debugStringVector(key->lin3Data()->value, prec));
+		item->setText(mKeyData1, debugStringVector(key->lin3Data()->dd, prec));
+		item->setText(mKeyData2, debugStringVector(key->lin3Data()->ds, prec));
 		break;
 	}
 	case Bo3DSTrackKey::TrackBool:
@@ -896,11 +898,11 @@ void BoTrackWidget::configureKey(QListViewItem* item, Bo3DSTrackKey* key, int ty
 	}
 	case Bo3DSTrackKey::TrackQuat:
 	{
-		item->setText(mKeyData0, key->quatData()->axis.debugString(prec));
+		item->setText(mKeyData0, debugStringVector(key->quatData()->axis, prec));
 		item->setText(mKeyData1, QString::number(key->quatData()->angle, 'f', prec));
-		item->setText(mKeyData2, key->quatData()->q.debugString(prec));
-		item->setText(mKeyData3, key->quatData()->dd.debugString(prec));
-		item->setText(mKeyData4, key->quatData()->ds.debugString(prec));
+		item->setText(mKeyData2, debugStringQuaternion(key->quatData()->q, prec));
+		item->setText(mKeyData3, debugStringQuaternion(key->quatData()->dd, prec));
+		item->setText(mKeyData4, debugStringQuaternion(key->quatData()->ds, prec));
 		break;
 	}
 	default:
@@ -909,10 +911,10 @@ void BoTrackWidget::configureKey(QListViewItem* item, Bo3DSTrackKey* key, int ty
  }
 }
 
-class BoMaterialWidget : public QWidget
+class BoMaterialDataWidget : public QWidget
 {
 public:
-	BoMaterialWidget(QWidget* parent) : QWidget(parent)
+	BoMaterialDataWidget(QWidget* parent) : QWidget(parent)
 	{
 		QVBoxLayout* l = new QVBoxLayout(this);
 		QGrid* grid = new QGrid(2, this);
@@ -989,7 +991,7 @@ public:
 		mWireSize = new QLabel(grid);
 	}
 
-	~BoMaterialWidget()
+	~BoMaterialDataWidget()
 	{
 	}
 
@@ -1170,8 +1172,8 @@ protected:
 		lib3ds_matrix_inv(invMeshMatrix);
 		matrix.loadMatrix(&invMeshMatrix[0][0]);
 
-		BoVector3 meshVertex[3];
-		BoVector3 v;
+		BoVector3Float meshVertex[3];
+		BoVector3Float v;
 		for (int j = 0; j < 3; j++) {
 			v.set(mesh->pointL[ face->points[j] ].pos);
 			matrix.transform(&meshVertex[j], &v);
@@ -1208,10 +1210,10 @@ protected:
 			item->setText(texelColumn[j], texel);
 		}
 
-		BoVector3 p, q;
+		BoVector3Float p, q;
 		p = meshVertex[2] - meshVertex[1];
 		q = meshVertex[0] - meshVertex[1];
-		BoVector3 normal = BoVector3::crossProduct(p, q);
+		BoVector3Float normal = BoVector3Float::crossProduct(p, q);
 		if (normal.length() != 0.0f) {
 			normal.normalize();
 		}
@@ -1310,7 +1312,7 @@ void BoNodeObjectDataWidget::setNodeObjectData(Lib3dsObjectData* d)
 	quatToAxisRotation(d->rot, &rX, &rY, &rZ, &angle);
 	rotAngle = QString("(%1,%2,%3) -> %4 degrees").arg(rX).arg(rY).arg(rZ).arg(angle);
 #endif
-	BoQuaternion q(d->rot[3], BoVector3(d->rot));
+	BoQuaternion q(d->rot[3], BoVector3Float(d->rot));
 	q.matrix().toRotation(&rX, &rY, &rZ);
 	rotX = QString::number(rX);
 	rotY = QString::number(rY);
@@ -1474,7 +1476,7 @@ public:
 	QWidget* mNodePage;
 
 	KListBox* mMaterialBox;
-	BoMaterialWidget* mMaterialData;
+	BoMaterialDataWidget* mMaterialData;
 	QPtrDict<Lib3dsMaterial> mListItem2Material;
 	KListView* mTextureView;
 
@@ -1556,7 +1558,7 @@ void KGameModelDebug::initMaterialPage()
  connect(d->mMaterialBox, SIGNAL(executed(QListBoxItem*)), this, SLOT(slotDisplayMaterial(QListBoxItem*)));
  QFontMetrics metrics(font());
 
- d->mMaterialData = new BoMaterialWidget(splitter);
+ d->mMaterialData = new BoMaterialDataWidget(splitter);
 
  d->mTextureView = new KListView(splitter);
  d->mTextureView->setAllColumnsShowFocus(true);
@@ -1599,6 +1601,7 @@ void KGameModelDebug::initMeshPage()
  d->mMeshView->addColumn(i18n("Faces count"), metrics.width(QString::number(111)));
  d->mMeshView->addColumn(i18n("Flags count"), metrics.width(QString::number(11)));
  d->mMeshView->addColumn(i18n("Max point index"), metrics.width(QString::number(11)));
+ d->mMeshView->addColumn(i18n("# of instances"), metrics.width(QString::number(11)));
  connect(d->mMeshView, SIGNAL(executed(QListViewItem*)), this, SLOT(slotDisplayMesh(QListViewItem*)));
  QVBox* modelInfo = new QVBox(meshView); // actually it doesn't fit to "meshView", but rather display info about the model
  QHBox* faces = new QHBox(modelInfo);
@@ -1908,6 +1911,21 @@ void KGameModelDebug::slotConstructMeshList()
 		indices = QMAX(indices, f->points[2]);
 	}
 	item->setText(6, QString::number(indices));
+	unsigned int instances = 0;
+	QPtrQueue<Lib3dsNode> allNodes;
+	for (Lib3dsNode* n = d->m3ds->nodes; n; n = n->next) {
+		allNodes.enqueue(n);
+	}
+	while (!allNodes.isEmpty()) {
+		Lib3dsNode* currentNode = allNodes.dequeue();
+		for (Lib3dsNode* n = currentNode->childs; n; n = n->next) {
+			allNodes.enqueue(n);
+		}
+		if ((currentNode->type == LIB3DS_OBJECT_NODE) && (strcmp(currentNode->name, mesh->name)) == 0) {
+			instances++;
+		}
+	}
+	item->setText(7, QString::number(instances));
 	d->mListItem2Mesh.insert(item, mesh);
  }
 }
