@@ -1,6 +1,7 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +23,10 @@
 #include <kplayer.h>
 #include <kgameproperty.h>
 
+#ifdef NO_PLAYER_H_HERE
+#error No player.h include is allowed here
+#endif
+
 class QColor;
 class QDomElement;
 class QTextStream;
@@ -34,7 +39,9 @@ class BosonMap;
 class BosonStatistics;
 class ProductionPlugin;
 class PlayerIO;
+class UpgradeProperties;
 
+class PlayerPrivate;
 /**
  * @author Thomas Capricelli <capricel@email.enst.fr>, Andreas Beckermann <b_mann@gmx.de>
  **/
@@ -46,7 +53,10 @@ public:
 		IdFogged = KGamePropertyBase::IdUser + 1,
 		IdMinerals = KGamePropertyBase::IdUser + 2,
 		IdOil = KGamePropertyBase::IdUser + 3,
-		IdIsNeutralPlayer = KGamePropertyBase::IdUser + 4
+		IdIsNeutralPlayer = KGamePropertyBase::IdUser + 4,
+		IdOutOfGame = KGamePropertyBase::IdUser + 5,
+		IdHasLost = KGamePropertyBase::IdUser + 6,
+		IdHasWon = KGamePropertyBase::IdUser + 7
 	};
 
 	/**
@@ -135,11 +145,24 @@ public:
 	 **/
 	bool isFogged(int x, int y) const;
 
+	/**
+	 * @return How many cells are currently fogged for this player
+	 **/
+	unsigned int foggedCells() const;
+
 	unsigned long int minerals() const;
 	unsigned long int oil() const;
 	void setMinerals(unsigned long int m);
 	void setOil(unsigned long int o);
 
+	void clearUpgrades();
+	void addUpgrade(const UpgradeProperties* upgrade);
+	void removeUpgrade(const UpgradeProperties* upgrade);
+	void removeUpgrade(unsigned long int id);
+	const QValueList<const UpgradeProperties*>* upgrades() const;
+
+	const UpgradeProperties* technologyProperties(unsigned long int type) const;
+
 	/**
 	 * Initialize the map for this player - this is mostly the fog of war,
 	 * currently.
@@ -170,17 +193,74 @@ public:
 	 * @return If the player is already "destroyed", i.e. doesn't have the
 	 * right to do anything. Note that this doesn't check if the player has
 	 * units left or something, but simply returns what was checked before.
-	 * Use @ref checkOutOfGame if you need an up-to-date information!
+	 * See also @ref setOutOfGame
 	 **/
 	bool isOutOfGame() const { return mOutOfGame; }
 
 	/**
-	 * Check if the player still fullfills the scenario conditions. If not
-	 * the player will be out of the game - i.e. from this point on @ref
-	 * isOutOfGame will always return true.
-	 * @return @ref isOutOfGame after it was updated.
+	 * The @ref hasWon and @ref hasLost flags are usually set at the ond of the
+	 * game, as information to the gameover dialog. They may however be set
+	 * earlier in the game to indicate that a player has already won (lost)
+	 * and cannot lose (win) anymore, no matter what he does. Note that this
+	 * means for example, that when the @ref hasWon flag is set, not even
+	 * losing all units could cause the player to lose the game. WARNING:
+	 * this interpretation of these flags (that they are permanent and
+	 * cannot change anymore once set in a game) might change in the future!
+	 *
+	 * Note that even at the end of the game it is possible that neither
+	 * @ref hasWon nor @ref hasLost are set. In that case the player is "in
+	 * between", he hasn't accomplished his goals, but hasn't lost the game
+	 * either. Winning and losing at once is not possible.
+	 *
+	 * @return TRUE if the player lost the game. During the game this is
+	 * usually not used directly, use @ref isOutOfGame instead.
 	 **/
-	bool checkOutOfGame();
+	bool hasLost() const { return mHasLost; }
+
+	/**
+	 * See @ref hasLost for a more detailed explanation of @ref hasLost and
+	 * @ref hasWon flags.
+	 *
+	 * @return TRUE if the player has won the game, otherwise FALSE. Note
+	 * that this is not necessarily the opposite of @ref hasLost, a player
+	 * may have neither won nor lost (however winning and losing is not
+	 * possible).
+	 **/
+	bool hasWon() const { return mHasWon; }
+
+	/**
+	 * See @ref hasLost
+	 **/
+	void setHasLost(bool l)
+	{
+		mHasLost = l;
+		if (hasLost() && hasWon()) {
+			// hasLost() has precedence
+			mHasWon = false;
+		}
+	}
+
+	/**
+	 * See @ref hasWon and @ref hasLost.
+	 **/
+
+	void setHasWon(bool w)
+	{
+		mHasWon = w;
+		if (hasLost()) {
+			// hasLost() has precedence
+			mHasWon = false;
+		}
+	}
+
+	/**
+	 * This is called by the global @ref BoCanvasEventListener to indicate
+	 * that the player has lost the game.
+	 *
+	 * The concrete moment of when this is called depends on the winning
+	 * conditions of the current map.
+	 **/
+	void setOutOfGame();
 
 	BosonStatistics* statistics() const;
 
@@ -226,7 +306,7 @@ public:
 	bool canResearchTech(unsigned long int id) const;
 
 	/**
-	 * @return TRUE if player has <em>constructed</em> (if it is a facility) unit 
+	 * @return TRUE if player has <em>constructed</em> (if it is a facility) unit
 	 * with type type, FALSE otherwise
 	 **/
 	bool hasUnitWithType(unsigned long int type) const;
@@ -249,8 +329,6 @@ signals:
 	void signalFog(int x, int y);
 	void signalUnfog(int x, int y);
 
-	void signalShowMiniMap(bool show);
-
 public slots:
 	/**
 	 * Called when a @ref KGameProperty object of a @ref Unit changes.
@@ -266,11 +344,12 @@ protected:
 	bool loadFogOfWar(const QDomElement& root);
 
 private:
-	class PlayerPrivate;
 	PlayerPrivate* d;
 
 	SpeciesTheme* mSpecies;
-	bool mOutOfGame;
+	KGameProperty<Q_UINT8> mOutOfGame;
+	KGameProperty<Q_UINT8> mHasLost;
+	KGameProperty<Q_UINT8> mHasWon;
 };
 
 #endif
