@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,8 +19,9 @@
 #ifndef PLUGINPROPERTIES_H
 #define PLUGINPROPERTIES_H
 
-#include <qstring.h>
+#include "boupgradeableproperty.h"
 
+#include <qstring.h>
 #include <qvaluelist.h>
 
 class UnitProperties;
@@ -47,7 +48,7 @@ class KSimpleConfig;
  * @short Base class for plugin properties for @ref UnitProperties
  * @authorAndreas Beckermann <b_mann@gmx.de>
  **/
-class PluginProperties
+class PluginProperties : public BoBaseValueCollection
 {
 public:
 	enum PluginPropertiesTypes {
@@ -77,6 +78,12 @@ public:
 	 **/
 	virtual int pluginType() const = 0;
 
+
+	/**
+	 * @return UnitProperties::upgradesCollection.
+	 **/
+	const BoUpgradesCollection& upgradesCollection() const;
+
 private:
 	const UnitProperties* mUnitProperties;
 };
