@@ -1,6 +1,8 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,11 +22,13 @@
 
 #include "unitbase.h"
 
+#include "../bomemory/bodummymemory.h"
 #include "unitproperties.h"
 #include "items/bosonitempropertyhandler.h"
 #include "player.h"
 #include "speciestheme.h"
 #include "bosoncanvas.h"
+#include "upgradeproperties.h"
 #include "bosonpropertyxml.h"
 #include "bodebug.h"
 
@@ -34,23 +38,25 @@
 #include <qmap.h>
 
 UnitBase::UnitBase(const UnitProperties* prop, Player* owner, BosonCanvas* canvas)
-	: BosonItem(owner->speciesTheme() ? owner->speciesTheme()->unitModel(prop->typeId()) : 0, canvas)
+	: BosonItem(owner, canvas),
+	mMaxHealth(prop, "Health", "MaxValue"),
+	mMaxArmor(prop, "Armor", "MaxValue"),
+	mMaxShields(prop, "Shields", "MaxValue"),
+	mMaxSightRange(prop, "SightRange", "MaxValue")
 {
  initStatic();
  mWeaponProperties = 0; // created on the fly in weaponDataHandler()
- mOwner = 0;
  mUnitProperties = prop; // WARNING: this might be 0 at this point! MUST be != 0 for Unit, but ScenarioUnit uses 0 here
- mIsMoving = false;
 
-// PolicyLocal?
- registerData(&mHealth, IdHealth);
- registerData(&mArmor, IdArmor);
- registerData(&mShields, IdShields);
  registerData(&mShieldReloadCounter, IdShieldReloadCounter);
- registerData(&mSightRange, IdSightRange);
+ registerData(&mDeletionTimer, IdDeletionTimer);
  registerData(&mWork, IdWork);
  registerData(&mAdvanceWork, IdAdvanceWork);
- registerData(&mDeletionTimer, IdDeletionTimer);
+ registerData(&mMovingStatus, IdMovingStatus);
+ registerData(&mHealthFactor, IdHealthFactor);
+ registerData(&mArmorFactor, IdArmorFactor);
+ registerData(&mShieldsFactor, IdShieldsFactor);
+ registerData(&mSightRangeFactor, IdSightRangeFactor);
 
  // these properties are fully internal (i.e. nothing is displayed in any
  // widget) and they change very often. So we increase speed greatly by not
@@ -58,14 +64,15 @@ UnitBase::UnitBase(const UnitProperties* prop, Player* owner, BosonCanvas* canva
  mDeletionTimer.setEmittingSignal(false);
  mShieldReloadCounter.setEmittingSignal(false);
 
- mWork.setLocal((int)WorkNone);
- mAdvanceWork.setLocal((int)WorkNone);
- mHealth.setLocal(0); // initially destroyed
- mShields.setLocal(0); // doesn't have any shields
+ mWork.setLocal((int)WorkIdle);
+ mAdvanceWork.setLocal((int)WorkIdle);
+ mMovingStatus.setLocal((int)Standing);
  mShieldReloadCounter.setLocal(0);
- mArmor.setLocal(0); // doesn't have any armor
- mSightRange.setLocal(0);
  mDeletionTimer.setLocal(0);
+ mHealthFactor.setLocal(0); // initially destroyed
+ mArmorFactor.setLocal(0); // doesn't have any armor
+ mShieldsFactor.setLocal(0); // doesn't have any shields
+ mSightRangeFactor.setLocal(0);
 }
 
 UnitBase::~UnitBase()
@@ -80,20 +87,38 @@ UnitBase::~UnitBase()
 // boDebug() << k_funcinfo << " done" << endl;
 }
 
+QString UnitBase::getModelIdForItem() const
+{
+ if (!owner()) {
+	BO_NULL_ERROR(owner());
+	return QString::null;
+ }
+ if (!owner()->speciesTheme()) {
+	BO_NULL_ERROR(owner()->speciesTheme());
+	return QString::null;
+ }
+ if (!unitProperties()) {
+	BO_NULL_ERROR(unitProperties());
+	return QString::null;
+ }
+ return QString("%1:%2").arg("unit").arg(QString::number(unitProperties()->typeId()));
+}
+
 void UnitBase::initStatic()
 {
  static bool initialized = false;
  if (initialized) {
 	return;
  }
- addPropertyId(IdHealth, QString::fromLatin1("Health"));
- addPropertyId(IdArmor, QString::fromLatin1("Armor"));
- addPropertyId(IdShields, QString::fromLatin1("Shields"));
- addPropertyId(IdSightRange, QString::fromLatin1("SightRange"));
  addPropertyId(IdWork, QString::fromLatin1("Work"));
  addPropertyId(IdAdvanceWork, QString::fromLatin1("AdvanceWork"));
  addPropertyId(IdDeletionTimer, QString::fromLatin1("DeletionTimer"));
  addPropertyId(IdShieldReloadCounter, QString::fromLatin1("ShieldReloadCounter"));
+ addPropertyId(IdMovingStatus, QString::fromLatin1("MovingStatus"));
+ addPropertyId(IdHealthFactor, QString::fromLatin1("HealthFactor"));
+ addPropertyId(IdArmorFactor, QString::fromLatin1("ArmorFactor"));
+ addPropertyId(IdShieldsFactor, QString::fromLatin1("ShieldsFactor"));
+ addPropertyId(IdSightRangeFactor, QString::fromLatin1("SightRangeFactor"));
  initialized = true;
 }
 
@@ -102,19 +127,92 @@ const QString& UnitBase::name() const
  return unitProperties()->name();
 }
 
+unsigned long int UnitBase::maxSightRange() const
+{
+ return mMaxSightRange.value(upgradesCollection());
+}
+
+unsigned long int UnitBase::sightRange() const
+{
+ return (unsigned long int)(sightRangeFactor() * maxSightRange());
+}
+
+void UnitBase::setSightRange(unsigned long int r)
+{
+ if (r > maxSightRange()) {
+	r = maxSightRange();
+ }
+ if (maxSightRange() > 0) {
+	setSightRangeFactor((bofixed)r / maxSightRange());
+ } else {
+	setSightRangeFactor(bofixed(1.0f));
+ }
+}
+
+unsigned long int UnitBase::maxArmor() const
+{
+ return mMaxArmor.value(upgradesCollection());
+}
+
 unsigned long int UnitBase::armor() const
 {
- return mArmor;
+ return (unsigned long int)(armorFactor() * maxArmor());
 }
 
 void UnitBase::setArmor(unsigned long int a)
 {
- mArmor = a;
+ if (a > maxArmor()) {
+	a = maxArmor();
+ }
+ if (maxArmor() > 0) {
+	setArmorFactor((bofixed)a / maxArmor());
+ } else {
+	setArmorFactor(bofixed(1.0f));
+ }
+}
+
+unsigned long int UnitBase::maxShields() const
+{
+ return mMaxShields.value(upgradesCollection());
+}
+
+unsigned long int UnitBase::shields() const
+{
+ return (unsigned long int)(shieldsFactor() * maxShields());
 }
 
 void UnitBase::setShields(unsigned long int s)
 {
- mShields = s;
+ if (s > maxShields()) {
+	s = maxShields();
+ }
+ if (maxShields() > 0) {
+	setShieldsFactor((bofixed)s / maxShields());
+ } else {
+	setShieldsFactor(bofixed(1.0f));
+ }
+}
+
+unsigned long int UnitBase::maxHealth() const
+{
+ return mMaxHealth.value(upgradesCollection());
+}
+
+unsigned long int UnitBase::health() const
+{
+ return (unsigned long int)(healthFactor() * maxHealth());
+}
+
+void UnitBase::setHealth(unsigned long int h)
+{
+ if (h > maxHealth()) {
+	h = maxHealth();
+ }
+ if (maxHealth() > 0) {
+	setHealthFactor((bofixed)h / maxHealth());
+ } else {
+	setHealthFactor(bofixed(1.0f));
+ }
 }
 
 unsigned long int UnitBase::type() const
@@ -122,22 +220,23 @@ unsigned long int UnitBase::type() const
  return unitProperties()->typeId();
 }
 
-void UnitBase::setOwner(Player* owner)
-{
- mOwner = owner;
-}
-
 bool UnitBase::saveAsXML(QDomElement& root)
 {
  if (!BosonItem::saveAsXML(root)) {
 	return false;
  }
  QDomDocument doc = root.ownerDocument();
- root.setAttribute(QString::fromLatin1("Rtti"), (int)rtti());
  root.setAttribute(QString::fromLatin1("Type"), (unsigned int)type());
  root.setAttribute(QString::fromLatin1("Group"), 0);
  root.setAttribute(QString::fromLatin1("GroupType"), 0);
 
+ QDomElement upgrades = doc.createElement("Upgrades");
+ root.appendChild(upgrades);
+ if (!upgradesCollection().saveAsXML(upgrades)) {
+	boError() << k_funcinfo << "Unable to save Upgrades" << endl;
+	return false;
+ }
+
  // save the weapon datahandler, if present
  if (mWeaponProperties) {
 	BosonCustomPropertyXML propertyXML;
@@ -148,6 +247,7 @@ bool UnitBase::saveAsXML(QDomElement& root)
 	}
 	root.appendChild(weaponHandler);
  }
+
  return true;
 }
 
@@ -161,6 +261,17 @@ bool UnitBase::loadFromXML(const QDomElement& root)
 	return false;
  }
 
+ clearUpgrades();
+ QDomElement upgrades = root.namedItem("Upgrades").toElement();
+ if (upgrades.isNull()) {
+	boError(260) << k_funcinfo << "NULL Upgrades tag" << endl;
+	return false;
+ }
+ if (!mUpgradesCollection.loadFromXML(speciesTheme(), upgrades)) {
+	boError() << k_funcinfo << "Unable to save Upgrades" << endl;
+	return false;
+ }
+
  // the weapon data handler
  QDomElement weaponHandler = root.namedItem(QString::fromLatin1("WeaponDataHandler")).toElement();
  if (!weaponHandler.isNull()) {
@@ -170,6 +281,7 @@ bool UnitBase::loadFromXML(const QDomElement& root)
 		return false;
 	}
  }
+
  return true;
 }
 
@@ -257,7 +369,7 @@ void UnitBase::reloadShields(int by)
  // reset to 10 and so on.
  // would allow configurable reload times more easily.
  if (mShieldReloadCounter >= MAX_SHIELD_RELOAD_COUNT) {
-	if (shields() < unitProperties()->shields()) {
+	if (shields() < maxShields()) {
 		setShields(shields() + 1);
 	}
 	mShieldReloadCounter = 0;
@@ -284,3 +396,40 @@ PlayerIO* UnitBase::ownerIO() const
  return owner()->playerIO();
 }
 
+const QValueList<const UpgradeProperties*>* UnitBase::upgrades() const
+{
+ return upgradesCollection().upgrades();
+}
+
+void UnitBase::clearUpgrades()
+{
+ mUpgradesCollection.clearUpgrades();
+}
+
+void UnitBase::addUpgrade(const UpgradeProperties* upgrade)
+{
+ BO_CHECK_NULL_RET(upgrade);
+ mUpgradesCollection.addUpgrade(upgrade);
+
+ if (!upgrade->upgradeUnit(this)) {
+	boError() << k_funcinfo << "error while applying upgrade to Unit " << id() << endl;
+	return;
+ }
+}
+
+void UnitBase::removeUpgrade(const UpgradeProperties* upgrade)
+{
+ if (!upgrade) {
+	return;
+ }
+ if (!mUpgradesCollection.removeUpgrade(upgrade)) {
+	// nothing changed
+	return;
+ }
+
+ if (!upgrade->downgradeUnit(this)) {
+	boError() << k_funcinfo << "error while un-applying upgrade from Unit " << id() << endl;
+	return;
+ }
+}
+
