@@ -1,6 +1,8 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,15 +23,14 @@
 
 #include "rtti.h"
 #include "items/bosonitem.h"
+#include "boupgradeableproperty.h"
 
 #include <kgame/kgameproperty.h>
 
-class QPoint;
-class QRect;
-class QDataStream;
 class KGamePropertyHandler;
 class QDomElement;
 template<class T, class T2> class QMap;
+template<class T> class QValueList;
 
 class Player;
 class PlayerIO;
@@ -38,12 +39,14 @@ class UnitProperties;
 class PluginProperties;
 class SpeciesTheme;
 class BosonItemPropertyHandler;
+class UpgradeProperties;
+
 
 /**
  * Ok, ok - this class is useless. All it does is providing a lot of properties
  * and the basic stuff of all units. But there is no point for an own class.
  * Maybe we'll put all of this directly to @ref Unit. But when I created the
- * basic design I still thought of some inheritance but never made this true. 
+ * basic design I still thought of some inheritance but never made this true.
  *
  * So here you can find most of the stuff about the unit, like health, weapon
  * damage, sight range and so on. But all the useful stuff is in @ref Unit.
@@ -62,25 +65,27 @@ public:
 	 * go the save way).
 	 **/
 	enum PropertyIds {
-		// properties in UnitBase
-		IdHealth = KGamePropertyBase::IdUser + 0,
-		IdArmor = KGamePropertyBase::IdUser + 1,
-		IdShields = KGamePropertyBase::IdUser + 2,
-		IdShieldReloadCounter = KGamePropertyBase::IdUser + 3,
-		IdSightRange = KGamePropertyBase::IdUser + 5,
-		IdWork = KGamePropertyBase::IdUser + 10,
-		IdAdvanceWork = KGamePropertyBase::IdUser + 11,
-		IdMovingStatus = KGamePropertyBase::IdUser + 12,
-		IdDeletionTimer = KGamePropertyBase::IdUser + 15,
-
-		// last entry.
-		IdLast
+		// UnitBase uses IDs from 512 to 1023.
+		//IdHealth = 512 + 0,
+		//IdArmor = 512 + 1,
+		//IdShields = 512 + 2,
+		IdShieldReloadCounter = 512 + 3,
+		//IdSightRange = 512 + 5,
+		IdWork = 512 + 10,
+		IdAdvanceWork = 512 + 11,
+		IdMovingStatus = 512 + 12,
+		IdDeletionTimer = 512 + 15,
+		IdHealthFactor = 512 + 20,
+		IdArmorFactor = 512 + 21,
+		IdShieldsFactor = 512 + 22,
+		IdSightRangeFactor = 512 + 23
 	};
 
 	/**
 	 * What is this unit currently doing
 	 *
 	 * Possible value are
+	 * @li WorkIdle - The unit has no specific orders
 	 * @li WorkNone - The unit does nothing
 	 * @li WorkMove A unit is currently moving
 	 * @li WorkAttack - Currently attacks a unit
@@ -91,14 +96,18 @@ public:
 	 * @li WorkTurn - is turning
 	 **/
 	enum WorkType {
-		WorkNone = 0,
+		// AB: only _append_ new numbers, do not insert something (when
+		// numbers get removed they still may be referenced by
+		// savegames)
+		WorkIdle = 0,
 		WorkMove = 2,
 		WorkAttack = 4,
 		WorkConstructed = 5,
 		WorkDestroyed = 6,
 		WorkFollow = 8,
 		WorkPlugin = 9,
-		WorkTurn = 10
+		WorkTurn = 10,
+		WorkNone = 11
 	};
 
 	/**
@@ -121,15 +130,17 @@ public:
 		Removing = 4,
 		MustSearch = 5
 	};
-	
+
 	UnitBase(const UnitProperties* prop, Player* owner, BosonCanvas* canvas);
 	virtual ~UnitBase();
 
+	virtual QString getModelIdForItem() const;
+
 	/**
 	 * Change what this unit is currently doing.
 	 **/
 	virtual void setWork(WorkType w)
-	{ 
+	{
 		mWork = w;
 		setAdvanceWork(w);
 	}
@@ -168,6 +179,8 @@ public:
 	 **/
 	const QString& name() const;
 
+	unsigned long int maxHealth() const;
+
 	/**
 	 * Health aka hitpoints
 	 *
@@ -178,12 +191,12 @@ public:
 	 * armor?
 	 * @return The health of the unit.
 	 **/
-	inline virtual unsigned long int health() const { return mHealth; };
+	virtual unsigned long int health() const;
 
 	/**
 	 * Change the health/power of this unit.
 	 **/
-	inline virtual void setHealth(unsigned long int h) { mHealth = h; }
+	virtual void setHealth(unsigned long int h);
 
 	bool isDestroyed() const
 	{
@@ -201,11 +214,13 @@ public:
 	 **/
 	KGamePropertyHandler* weaponDataHandler();
 
-	inline unsigned long int shields() const { return mShields; }
+	unsigned long int shields() const;
 	void setShields(unsigned long int shields);
+	unsigned long int maxShields() const;
 
 	unsigned long int armor() const;
 	void setArmor(unsigned long int armor);
+	unsigned long int maxArmor() const;
 
 	/**
 	 * The type of the unit as described in the index.unit file of this
@@ -219,11 +234,11 @@ public:
 	inline virtual int rtti() const { return RTTI::UnitStart + (int)type(); }
 
 	/**
-	 * @return How far this unit can see. This is a number of cells, so you
-	 * must *= BO_TILE_SIZE to use this on the canvas.
+	 * @return How far this unit can see. This is a number of cells
 	 **/
-	inline unsigned int sightRange() const { return mSightRange; }
-	void setSightRange(unsigned int r) { mSightRange = r; }
+	unsigned long int sightRange() const;
+	virtual void setSightRange(unsigned long int r);
+	unsigned long int maxSightRange() const;
 
 	virtual bool saveAsXML(QDomElement& root);
 	virtual bool loadFromXML(const QDomElement& root);
@@ -240,7 +255,7 @@ public:
 	 * care about crashes as the game will crash anyway.
 	 **/
 	const UnitProperties* unitProperties() const { return mUnitProperties; }
-	
+
 	/**
 	 * Convenience method for unitProperties()->properties().
 	 *
@@ -274,7 +289,7 @@ public:
 	 * airport. For these units we need to know whether they are flying or
 	 * not.
 	 * @return Whether this unit is currently flying. Always false if @ref
-	 * unitProperties()->isAircraft is false. Currently this is juste the 
+	 * unitProperties()->isAircraft is false. Currently this is juste the
 	 * same as @ref unitProperties()->isAircraft.
 	 **/
 	bool isFlying() const;
@@ -303,6 +318,16 @@ public:
 	 **/
 	virtual bool saveScenario(QDomElement& node);
 
+	const QValueList<const UpgradeProperties*>* upgrades() const;
+
+	void clearUpgrades();
+	virtual void addUpgrade(const UpgradeProperties* upgrade);
+	virtual void removeUpgrade(const UpgradeProperties* upgrade);
+	const BoUpgradesCollection& upgradesCollection() const
+	{
+		return mUpgradesCollection;
+	}
+
 protected:
 	/**
 	 * Should get called in every @ref Unit::advance call. This counts the
@@ -313,25 +338,47 @@ protected:
 	 **/
 	void reloadShields(int by = 1);
 
+	/**
+	 * @return how many percentage of maximum health this unit has
+	 * Note that the returned value is in range 0-1, not 0-100!
+	 **/
+	inline bofixed healthFactor() const { return mHealthFactor; }
+	inline void setHealthFactor(bofixed f) { mHealthFactor = f; }
+
+	inline bofixed armorFactor() const { return mArmorFactor; }
+	inline void setArmorFactor(bofixed f) { mArmorFactor = f; }
+
+	inline bofixed shieldsFactor() const { return mShieldsFactor; }
+	inline void setShieldsFactor(bofixed f) { mShieldsFactor = f; }
+
+	inline bofixed sightRangeFactor() const { return mSightRangeFactor; }
+	inline void setSightRangeFactor(bofixed f) { mSightRangeFactor = f; }
+
 private:
 	static void initStatic();
 
 private:
-	KGameProperty<unsigned long int> mArmor;
-	KGameProperty<unsigned long int> mShields;
+	const UnitProperties* mUnitProperties;
+	BoUpgradesCollection mUpgradesCollection;
+
+	BosonItemPropertyHandler* mWeaponProperties;
+
 	KGameProperty<unsigned long int> mShieldReloadCounter;
 	KGameProperty<unsigned int> mDeletionTimer;
-	KGameProperty<unsigned long int> mHealth;
-	KGameProperty<unsigned int> mSightRange;
 	KGamePropertyInt mWork;
 	KGamePropertyInt mAdvanceWork;
 	KGamePropertyInt mMovingStatus;
 
-	BosonItemPropertyHandler* mWeaponProperties;
+	KGameProperty<bofixed> mHealthFactor;
+	KGameProperty<bofixed> mArmorFactor;
+	KGameProperty<bofixed> mShieldsFactor;
+	KGameProperty<bofixed> mSightRangeFactor;
 
-	const UnitProperties* mUnitProperties;
 
-	friend class UpgradePropertiesBase;
+	BoUpgradeableProperty<unsigned long int> mMaxHealth;
+	BoUpgradeableProperty<unsigned long int> mMaxArmor;
+	BoUpgradeableProperty<unsigned long int> mMaxShields;
+	BoUpgradeableProperty<unsigned long int> mMaxSightRange;
 };
 
 #endif
